@@ -18,7 +18,7 @@ convert big_logo.png -resize 100x100 title_logo.png
 
 
 echo -n "image_list = " > $MFILE
-for S in "*.png" "*.jpg" "*.svg"; do
+for S in "*.png" "*.jp*g" "*.svg"; do
     for A in $S; do
 	echo "\\" >> $MFILE
 	echo -n " $A " >> $MFILE
