@@ -9,12 +9,12 @@ munipack dark d120_*.fits
 munipack phcorr -gain 2.3 -dark dark.fits ngc7635_*.fits
 
 # find stars, aperture photometry
-munipack find ngc7635_*.fits
-munipack aphot ngc7635_*.fits
+munipack find ngc7635_*_proc.fits
+munipack aphot ngc7635_*_proc.fits
 
 # astrometry
 munipack cone -r 0.1 350.20 61.20
-munipack astrometry --maxmatch 20 ngc7635_*.fits
+munipack astrometry ngc7635_*_proc.fits
 
 # kombine
-munipack kombine -o bubble.fits ngc7635_*.fits
+munipack kombine -o bubble.fits ngc7635_*_proc.fits
