@@ -27,17 +27,18 @@ feed = fd.Atom1Feed(title,link,description,author_name="Filip Hroch",
                     author_email="hroch@physics.muni.cz")
 
 records = [
-    record("0.5.0",link,"Start of New Generation",2010,02,10),
-    record("0.5.1",link,"Coloring release",2010,10,17),
-    record("0.5.2",link,"Astrometry release",2011,10,19),
-    record("0.5.3",link,"New generation release",2012,06,29),
-    record("0.5.4",link,"Photon Rain",2013,07,31),
-    record("0.5.5",link,"Nights Without Convergence",2014,04,01),
-    record("0.5.6",link,"Photon Calibration",2015,01,05),
-    record("0.5.7",link,"Growth Curve",2016,10,25),
-    record("0.5.8",link,"Artificial Sky",2017,07,14),
-    record("0.5.9",link,"Artificial Sky",2017,07,18),
-    record("0.5.10",link,"Flat-fielding",2018,01,16)
+   record("0.5.0",link,"Start of New Generation",2010,02,10),
+   record("0.5.1",link,"Coloring release",2010,10,17),
+   record("0.5.2",link,"Astrometry release",2011,10,19),
+   record("0.5.3",link,"New generation release",2012,06,29),
+   record("0.5.4",link,"Photon Rain",2013,07,31),
+   record("0.5.5",link,"Nights Without Convergence",2014,04,01),
+   record("0.5.6",link,"Photon Calibration",2015,01,05),
+   record("0.5.7",link,"Growth Curve",2016,10,25),
+   record("0.5.8",link,"Artificial Sky",2017,07,14),
+   record("0.5.9",link,"Artificial Sky",2017,07,18),
+   record("0.5.10",link,"Flat-fielding",2018,01,16),
+   record("0.5.11",link,"Oak Leaf",2019,01,22)
 ]
 
 for r in records:
