@@ -24,9 +24,7 @@
 #include "munipack.h"
 #include "vocatconf.h"
 #include <wx/wx.h>
-#include <wx/log.h>
 #include <wx/filename.h>
-#include <wx/tokenzr.h>
 
 using namespace std;
 
