@@ -2,7 +2,7 @@
 
   Astrometry
 
-  Copyright © 2010 - 2017 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2010 - 2018 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -38,10 +38,6 @@
 #include "vocatconf.h"
 #include <wx/wx.h>
 #include <wx/tokenzr.h>
-#include <wx/regex.h>
-#include <wx/sstream.h>
-#include <wx/txtstrm.h>
-#include <wx/filename.h>
 
 bool Munipack::astrometry(MuniProcess *action, MuniCmdLineParser& cmd)
 {
@@ -200,9 +196,7 @@ bool Munipack::astrometry(MuniProcess *action, MuniCmdLineParser& cmd)
 
   }
 
-
-  for(size_t i = 0; i < cmd.GetParamCount(); i++)
-    WriteFiles(action,cmd.GetParam(i));
+  WriteFiles(action,cmd);
 
   return true;
 
