@@ -2,7 +2,7 @@
 
   Munipack - command line parser
 
-  Copyright © 2010-17 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2010-19 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -19,7 +19,6 @@
   You should have received a copy of the GNU General Public License
   along with Munipack.  If not, see <http://www.gnu.org/licenses/>.
 
-
 */
 
 #include "munipack.h"
@@ -27,58 +26,72 @@
 #include <wx/app.h>
 #include <wx/cmdline.h>
 #include <wx/regex.h>
-
-#define COMMON_OPTIONS_BACKUP \
-  AddSwitch("b","","make backup (default: on)");  \
-  AddOption("","backup","make and set backup method (default: existing)"); \
-  AddOption("S","suffix","suffix for backup (default: ~)");
+#include <wx/tokenzr.h>
+#include <wx/filename.h>
+#include <wx/file.h>
+#include <wx/wfstream.h>
+#include <wx/txtstrm.h>
 
 #define COMMON_OPTIONS_SINGLE \
-  AddOption("o","output","output file, (-) for stdout");	\
-  COMMON_OPTIONS_BACKUP
+  AddOption("o","output","output file");
+
+#define COMMON_OPTIONS_CLOBBER	\
+  AddSwitch("","no-clobber","do not overwrite existing files");
 
 #define COMMON_OPTIONS_MULTIPLE \
-  COMMON_OPTIONS_BACKUP \
   AddOption("t","target-directory","set target directory");		\
-  AddSwitch("O",wxEmptyString,"switch-on advanced output filenames processing"); \
-  AddOption(wxEmptyString,"pattern","matching pattern (for -O)");	\
-  AddOption(wxEmptyString,"mask","mask for output file(s) (for -O)");	\
-  AddOption(wxEmptyString,"format","format for output files (for -O)");
+  AddSwitch("O","","switch-on advanced output filenames processing");\
+  AddOption("","pattern","matching pattern (for -O)");	\
+  AddOption("","mask","mask for output file(s) (for -O)");	\
+  AddOption("","format","format for output files (for -O)");
 
 #define COMMON_OPTIONS_BITPIX \
   AddOption("B","bitpix","bitpix (-32,32,16,8)",wxCMD_LINE_VAL_NUMBER);
 
-#define COMMON_OPTIONS_PHTOLLS \
-  AddOption("","tol-temp",L"limit for nearby temperatures (1°C)",wxCMD_LINE_VAL_DOUBLE);\
-  AddOption("","tol-exp",L"limit for nearby exposure times (1μsec)",wxCMD_LINE_VAL_DOUBLE);
-
-#define COMMON_OPTIONS_ARITH \
-  AddSwitch(wxEmptyString,"arith","arithmetical mean instead of robust mean");
-
-#define COMMON_OPTIONS_ADDKEY \
-  AddOption("K","keys","key=value[,key=,..] add keyword(s) to header",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
+#define COMMON_OPTIONS_PHCORR \
+  AddOption("bitmask",wxEmptyString,"mask frame"); \
+  AddOption("st","saturate","saturation in counts",wxCMD_LINE_VAL_DOUBLE); \
+  AddOption("th","threshold","threshold in counts",wxCMD_LINE_VAL_DOUBLE);
 
 #define COMMON_OPTIONS_PHQ \
     AddOption("q","quantity","select: PHRATE,FLUX,FNU,FLAM,MAG,ABMAG,STMAG");
 //    AddOption("q","quantity","instrumental: COUNT,RATE,MAG\n\tcalibrated: PHOTON,PHOTRATE,PHOTNU,PHOTLAM,FLUX,FNU,FLAM,MAG,ABMAG,STMAG");
 
 
-MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t *xargv[]):
+MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc,
+				     wchar_t *xargv[]):
   wxCmdLineParser(), argc(0), argv(0)
 {
   if( xargc > 0 && action != "" ) {
-    // filter command-line options:
-    // * actions are not passed
 
-    argc = xargc - 1;
+    // filters command-line options:
+    // * actions are not passed
+    // * remove stdin "-" switch
+    // * remove @in item
+    // * read input files and defines files
+
+    // Look for - or @in option and eventually read the input files
+    int larg = 0;
+    for(int i = 0; i < xargc; i++ ) {
+      wxString arg(xargv[i]);
+      if( arg.substr(0,1) == "@" || arg == "-" ) {
+	if( InputByFile(arg.substr(1)) )
+	  larg++;
+	else
+	  wxLogFatalError("Failed to read input files from `" + arg + "'.");
+      }
+    }
+
+    // Update arguments while removing unwanted parameters
+    argc = xargc - 1 - larg;
     argv = new wxChar*[argc];
     int n = 0;
     for(int i = 0; i < xargc; i++) {
-      if( xargv[i] != action ) {
-	wxString a(xargv[i]);
-	argv[n++] = wxStrdup(a.wc_str());
-      }
+      wxString arg(xargv[i]);
+      if( arg != action && arg != "-" &&  arg.substr(0,1) != "@" )
+	argv[n++] = wxStrdup(arg.wc_str());
     }
+    wxASSERT(n == argc);
 
     SetCmdLine(argc,argv);
   }
@@ -91,14 +104,13 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
 		 " calibrate\tcalibrate\n"
 		 */
 		 "\nActions:\n"
-		 " phcorr\t\tApply photometric corrections\n"
 		 " bias\t\tAveraged bias frame\n"
 		 " dark\t\tAveraged dark frame\n"
 		 " flat\t\tAveraged flat-field frame\n"
-		 " coloring\tCompose colour frame\n"
-		 " kombine\tCompose frames (deep exposures, mosaics)\n"
-		 " ctrafo\t\tTransform colour frames between colour-spaces\n"
-		 " cone\t\tCone search in a catalogue on Virtual Observatory\n"
+		 " phcorr\t\tApply photometric corrections\n"
+		 " colouring\tCompose colour frames\n"
+		 " kombine\tCombine frames (deep exposures, mosaics)\n"
+		 " cone\t\tCone search in a Virtual Observatory catalogue\n"
 		 " astrometry\tAstrometry calibration\n"
 		 " find\t\tSearch for stars on frames\n"
 		 " aphot\t\tAperture photometry\n"
@@ -117,7 +129,6 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
     AddParam("action",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_OPTIONAL);
 
     SetCmdLine(xargc,xargv);
-
     return;
   }
 
@@ -129,23 +140,22 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
 
     AddUsageText("Usage: munipack "+action+" [options] file(s)");
     AddUsageText("* Averaged bias frame.");
-    COMMON_OPTIONS_ARITH
-    COMMON_OPTIONS_PHTOLLS
-    COMMON_OPTIONS_BITPIX
+    COMMON_OPTIONS_PHCORR
     COMMON_OPTIONS_SINGLE
-    AddParam("file(s)",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
+    COMMON_OPTIONS_BITPIX
+    AddParam("file(s)",wxCMD_LINE_VAL_STRING,
+	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
   }
   else if( action == "dark" ) {
 
     AddUsageText("Usage: munipack "+action+" [options] file(s)");
     AddUsageText("* Averaged dark frame.");
-
-    AddOption("bias",wxEmptyString,"bias frame");
-    COMMON_OPTIONS_ARITH
-    COMMON_OPTIONS_PHTOLLS
-    COMMON_OPTIONS_BITPIX
+    AddOption("bias","","bias frame");
+    COMMON_OPTIONS_PHCORR
     COMMON_OPTIONS_SINGLE
-    AddParam("file(s)",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
+    COMMON_OPTIONS_BITPIX
+    AddParam("file(s)",wxCMD_LINE_VAL_STRING,
+	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
   }
 
   else if( action == "flat" ) {
@@ -153,69 +163,64 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
     AddUsageText("Usage: munipack "+action+" [options] file(s)");
     AddUsageText("* Averaged flat-field frame.");
 
-    AddOption("gain","","provide value of gain (rather than FITS keyword)",
+    AddOption("gain","","provide value of gain (rather by FITS header)",
 	      wxCMD_LINE_VAL_DOUBLE);
     AddOption("bias",wxEmptyString,"bias frame");
     AddOption("dark",wxEmptyString,"dark frame");
     AddOption("xdark",wxEmptyString,"dark frame exposure time factor",
 	      wxCMD_LINE_VAL_DOUBLE);
-    AddOption("bitmask",wxEmptyString,"mask frame");
-    AddOption("st","saturate","saturation in counts",wxCMD_LINE_VAL_DOUBLE);
-    AddOption("th","threshold","threshold in counts",wxCMD_LINE_VAL_DOUBLE);
-    AddSwitch("","gain-estimate","rough estimate of gain");
-    AddOption("","approximation","precision: basic, standard (default)");
-    COMMON_OPTIONS_BITPIX
+    COMMON_OPTIONS_PHCORR
+    AddOption("","approximation","accuracy: basic, standard (default)");
     COMMON_OPTIONS_SINGLE
-    AddParam("file(s)",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
+    COMMON_OPTIONS_BITPIX
+    AddParam("file(s)",wxCMD_LINE_VAL_STRING,
+	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
   }
 
   else if( action == "phcorr" ) {
 
-    AddUsageText("Usage: munipack "+action+" [options] file(s)[,result(s)]");
+    AddUsageText("Usage: munipack "+action+" [options] file(s)");
     AddUsageText("* Apply photometric corrections (gain, bias, dark, flat-field).");
 
     AddOption("gain","","provide value of gain (rather than FITS keyword)",
 	      wxCMD_LINE_VAL_DOUBLE);
-    AddOption("flat",wxEmptyString,"flat-field frame");
-    AddOption("bias",wxEmptyString,"bias frame");
-    AddOption("dark",wxEmptyString,"dark frame");
-    AddOption("xdark",wxEmptyString,"dark frame exposure time factor",
+    AddOption("flat","","flat-field frame");
+    AddOption("bias","","bias frame");
+    AddOption("dark","","dark frame");
+    AddOption("xdark","","dark frame exposure time factor",
 	      wxCMD_LINE_VAL_DOUBLE);
-    AddOption("bitmask",wxEmptyString,"mask frame");
-    AddOption("xbitmask",wxEmptyString,"mask frame as ZERO (default), MEAN (average)");
-    AddSwitch("","gain-ignore","ignore gain transformation");
-    AddSwitch(wxEmptyString,"normalise","normalise flat-field prior to use");
-    AddSwitch(wxEmptyString,"force","apply to already corrected frames");
-    COMMON_OPTIONS_PHTOLLS
-    COMMON_OPTIONS_BITPIX
+    AddOption("bitmask","","mask frame");
+    AddOption("xbitmask","","mask frame as ZERO, MEAN(default), MEDIAN");
+    AddOption("box","","box size for MEDIAN bitmask",wxCMD_LINE_VAL_NUMBER);
+    AddSwitch("","normalise-flat","normalise flat-field prior to use");
+    COMMON_OPTIONS_CLOBBER
     COMMON_OPTIONS_MULTIPLE
-    AddParam("file(s)[,result(s)]",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
+    COMMON_OPTIONS_BITPIX
+    AddParam("file(s)",wxCMD_LINE_VAL_STRING,
+	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
   }
 
-  else if( action == "coloring" ) {
+  else if( action == "colouring" ) {
 
-    AddUsageText("Usage: munipack "+action+" [options] file(s)[,ext(s)]");
-    AddUsageText("* Compose colour frame.");
+    AddUsageText("Usage: munipack "+action+" [options] files");
+    AddUsageText("* Compose colour frames");
 
-    AddOption("c",wxEmptyString,"colorspace");
-    AddOption("ct",wxEmptyString,"colour table");
-    COMMON_OPTIONS_BITPIX
+    AddOption("c","cspace-input","input colour-space, default: ask oracle");
+    AddOption("","cspace-output","output colour-space, default:'CIE 1931 XYZ'");
+    AddOption("w","weights","w1,w2,.. weights of images");
+    AddOption("q","ctphs","ctph1,ctph2,.. ctphs of images");
+    AddOption("b","backs","b1,b2,.. backgrounds of images");
+    AddSwitch("","disable-back","don't estimate background of images");
+    AddOption("","white-spot","x,y coordinates of centre of white spot");
+    AddOption("","white-star","x,y coordinates of centre of white star");
+    AddOption("","white-radius","common for both white spot or star, default:7",
+	      wxCMD_LINE_VAL_DOUBLE);
+    AddSwitch("","list","available colour-space transformations");
+    AddOption("","ctable","colour table transformation definition");
+    AddOption("","phsystab","photometry systems definition table");
     COMMON_OPTIONS_SINGLE
-    AddParam("files[,exts]",wxCMD_LINE_VAL_STRING,
-  	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
-  }
-
-  else if( action == "ctrafo" ) {
-
-    AddUsageText("Usage: munipack "+action+" [options] file black(s),weight(s)");
-    AddUsageText("* Transform colour frames between various colour-spaces.");
-
-    AddSwitch(wxEmptyString,"list","known collapses");
-    AddOption("c",wxEmptyString,"collapse");
-    AddOption("ct",wxEmptyString,"colour table");
     COMMON_OPTIONS_BITPIX
-    COMMON_OPTIONS_SINGLE
-    AddParam("file black,weight",wxCMD_LINE_VAL_STRING,
+    AddParam("files",wxCMD_LINE_VAL_STRING,
   	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
   }
 
@@ -225,8 +230,13 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
     AddUsageText("* Cone search in catalogues of Virtual Observatory.");
 
     AddOption("c","cat","catalogue (see --list-catalogues)");
-    AddOption("r","radius","radius for cone search [deg] (default: 6 arcmin = 0.1)",
+    AddOption("r","radius",
+	      "radius for cone search [deg] (default: 6 arcmin = 0.1)",
 	      wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","magmin","low magnitude limit",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","magmax","high magnitude limit",wxCMD_LINE_VAL_DOUBLE);
+    AddSwitch("","Johnson-patch",
+	      "convert Gunn's ri magnitudes to Johnson RI (UCAC4 only)");
     AddOption("s","sort","sort by the column");
     AddOption("","par","add optional parameters");
     AddOption("","url","service URL");
@@ -235,7 +245,7 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
     AddSwitch("","list-catalogues","show available catalogues");
     AddSwitch("","list-servers","show available VO servers");
     AddOption("","vocat","configuration file");
-    AddOption("","type","type of output file: fits,csv,txt,xml");
+    AddOption("","type","type of output file: fits,xml");
     COMMON_OPTIONS_SINGLE
     AddParam("ra",wxCMD_LINE_VAL_DOUBLE,wxCMD_LINE_PARAM_OPTIONAL);
     AddParam("dec",wxCMD_LINE_VAL_DOUBLE,wxCMD_LINE_PARAM_OPTIONAL);
@@ -247,7 +257,6 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
     AddUsageText("* Conversions of VOTables.");
 
     AddOption("","type","type of output file: fits,csv,txt,svg,xml");
-    AddOption("s","","sort by the column");
     AddOption("pt","","select projection: gnomon");
     AddOption("pa","","Right Ascension of projection centre",wxCMD_LINE_VAL_DOUBLE);
     AddOption("pd","","Declination of projection centre",wxCMD_LINE_VAL_DOUBLE);
@@ -325,31 +334,36 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
 
   else if( action == "find" ) {
 
-    AddUsageText("Usage: munipack "+action+" [options] file(s)[,results(s)]");
+    AddUsageText("Usage: munipack "+action+" [options] file(s)");
     AddUsageText("* Search for stars on frames.");
 
     AddOption("f","fwhm","FWHM in pixels",wxCMD_LINE_VAL_DOUBLE);
-    AddOption("th","threshold","threshold in sigmas above background",wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"saturate","saturation in ADU",wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"lothresh","lower for threshold in sigmas",wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"rndlo","lower for round",wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"rndhi","higher for round",wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"shrplo","lower for sharp",wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"shrphi","higher for sharp",wxCMD_LINE_VAL_DOUBLE);
-    AddOption("","read-noise","read noise in ADU (default: frame header)",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("th","threshold","threshold in sigmas above background",
+	      wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","saturate","saturation in ADU",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","lothresh","lower for threshold in sigmas",
+	      wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","rndlo","lower for round",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","rndhi","higher for round",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","shrplo","lower for sharp",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","shrphi","higher for sharp",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","read-noise","read noise in ADU (default: frame header)",
+	      wxCMD_LINE_VAL_DOUBLE);
     COMMON_OPTIONS_MULTIPLE
-    AddParam("file(s)[,results(s)]",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
+    AddParam("file(s)",wxCMD_LINE_VAL_STRING,
+	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
   }
 
   else if( action == "aphot" ) {
 
-    AddUsageText("Usage: munipack "+action+" [options] file(s)[,results(s)]");
+    AddUsageText("Usage: munipack "+action+" [options] file(s)");
     AddUsageText("* Aperture photometry");
 
-    AddOption(wxEmptyString,"apertures","a1,a2,... define aperture radii in pixels ");
-    AddOption(wxEmptyString,"ring","ri,ro inner and outer sky ring radius in pixels ");
+    AddOption("","apertures","a1,a2,... define aperture radii in pixels");
+    AddOption("","ring","ri,ro inner and outer sky ring radius in pixels");
     COMMON_OPTIONS_MULTIPLE
-    AddParam("file(s)[,results(s)]",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
+    AddParam("file(s)",wxCMD_LINE_VAL_STRING,
+	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
   }
 
   else if( action == "gphot" ) {
@@ -373,24 +387,32 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
 
   else if( action == "timeseries" ) {
 
-    AddUsageText("Usage: munipack "+action+" [options] [RA,Dec] ... file(s)");
-    AddUsageText("* List anything to time-series.");
+    AddUsageText("Usage: munipack "+action+" [options] ... [file(s)]");
+    AddUsageText("* List keywords or objects to time-series.");
 
-    AddOption("c","column","column name(s) to list");
-    AddOption("C","coocolumn","column name(s) of coordinates (object search)");
-    AddOption("x","extname","FITS extension identification");
-    AddOption("T","time-type","JD (default), MJD, HJD, PHASE");
-    AddOption(wxEmptyString,"time-stamp","MID (default), BEGIN, END");
-    AddOption(wxEmptyString,"time-err","uncertainty in the time measurement [s]",wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"time-res","resolution in the time measurement [s]",wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"epoch","reference time for light curve elements [JD]",wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"period","period light curve elements [days]",wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"tol","search radius [deg]",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("c","coo","coordinates of objects");
+    AddOption("","cat","coordinates by FITS catalogue");
+    AddOption("l","col","column label(s) to list");
+    AddOption("K","keys","key[,key] header key(s) to list");
+    AddOption("T","time-type","time types:  JD (default), MJD, HJD, PHASE");
+    AddOption("","time-stamp","MID (default), BEGIN, END");
     AddOption("","coo-type","DEG (default), RECT");
-    AddOption(wxEmptyString,"cat","coordinates from the FITS file");
-    AddSwitch(wxEmptyString,"stdout","print results also to standard output");
+    AddOption("","coo-col","column label(s) of coordinates (object search)");
+    AddOption("","tol","crosmatch coordinate search limit [deg]",
+	      wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","lc-epoch","reference time for light curve elements [JD]",
+	      wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","lc-period","period for light curve elements [days]",
+	      wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","extname","specify FITS extension name");
+    AddSwitch("","stdout","list results to standard output");
+    AddSwitch("","enable-filename","print filename");
+    AddSwitch("","disable-timetype","disable print of time-type quantity");
+    AddSwitch("","enable-horizon","print horizontal coordinates");
+    AddSwitch("","enable-airmass","print air-mass");
     COMMON_OPTIONS_SINGLE
-    AddParam("[a,d]... file(s)",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
+    AddParam("[file(s)]",wxCMD_LINE_VAL_STRING,
+	     wxCMD_LINE_PARAM_OPTIONAL | wxCMD_LINE_PARAM_MULTIPLE);
   }
 
   // case ID_LIST:
@@ -438,22 +460,28 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
 
   else if( action == "fits" ) {
 
-    AddUsageText("Usage: munipack "+action+" [options] file(s)[,result(s)]");
+    AddUsageText("Usage: munipack "+action+" [options] file(s)");
     AddUsageText("* Operations on FITS files.");
 
     AddSwitch("lh","header","list header");
     AddSwitch("lt","table","list table");
     AddSwitch("li","image","list image");
-    AddOption("K","print-keys","key[,key,..] print selected header keyword(s)");
-    AddOption(wxEmptyString,"remove-keys","key[,key,..] remove keyword(s)");
-    AddOption(wxEmptyString,"update-key","key=value[,comment] add/update keyword");
-    AddOption(wxEmptyString,"remove-extensions","extname[,extname,..] remove extension(s)");
-    AddSwitch(wxEmptyString,"shell","shell-friendly format for print of keywords");
-    AddSwitch(wxEmptyString,"value","only values of keywords are printed");
-    AddSwitch(wxEmptyString,"dump","dump FITS to plain text");
-    AddSwitch(wxEmptyString,"restore","restore FITS from plain text");
+    AddOption("K","keys","key[,key,..] print header values by keyword(s)");
+    AddSwitch("","value","print only values during keywords print");
+    AddSwitch("","shell","shell-friendly format of keywords print");
+    AddSwitch("","update","add or update header record");
+    AddOption("","key","specify keyword of updated record");
+    AddOption("","val","specify value of updated record");
+    AddOption("","com","specify comment of updated record");
+    AddOption("","templ","update records by this file");
+    AddOption("","remove-keys","key[,key,..] remove keyword(s)");
+    AddOption("","remove-extensions","extname[,extname,..] remove extension(s)");
+    AddSwitch("","dump","dump FITS to plain text");
+    AddSwitch("","restore","restore FITS from plain text");
+    AddSwitch("","cat","cat (copy) input on output");
     COMMON_OPTIONS_MULTIPLE
-    AddParam("file(s)[,result(s)]",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
+    AddParam("[file(s)]",wxCMD_LINE_VAL_STRING,
+	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
   }
 
   else if( action == "cross" ) {
@@ -477,30 +505,31 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
     AddUsageText("* Compose frames (deep exposures, mosaics).");
 
     AddOption("p","projection","projection: identity, gnomonic");
-    AddOption(wxEmptyString,"xcen","centre of frame [pix] (default: width/2)",
+    AddOption("","xcen","centre of frame [pix] (default: width/2)",
 	      wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"ycen","centre of frame [pix] (default: height/2)",
+    AddOption("","ycen","centre of frame [pix] (default: height/2)",
 	      wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"rcen","centre of FOV in Right Ascension [deg]",
+    AddOption("","rcen","centre of FOV in Right Ascension [deg]",
 	      wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"dcen","centre of FOV in Declination [deg]",
+    AddOption("","dcen","centre of FOV in Declination [deg]",
 	      wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"width","width of output [pix]",
-	      wxCMD_LINE_VAL_NUMBER);
-    AddOption(wxEmptyString,"height","height of output [pix]",
-	      wxCMD_LINE_VAL_NUMBER);
-    AddOption(wxEmptyString,"scale","scale [pix/deg]",
+    AddOption("","pm-ra","proper motion in Right Ascension [deg/day]",
 	      wxCMD_LINE_VAL_DOUBLE);
-    AddOption(wxEmptyString,"angle","position angle [deg]",
+    AddOption("","pm-dec","proper motion in Declination [deg/day]",
 	      wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","pm-jdref","proper motion reference JD",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","width","width of output [pix]",wxCMD_LINE_VAL_NUMBER);
+    AddOption("","height","height of output [pix]",wxCMD_LINE_VAL_NUMBER);
+    AddOption("","scale","scale [deg/pix]",wxCMD_LINE_VAL_DOUBLE);
+    AddOption("","angle","position angle [deg]",wxCMD_LINE_VAL_DOUBLE);
     AddOption("","reflex","[yes|no] set/unset reflection");
-    AddOption("i","interpol","interpolation: none,bilinear,bicubic"); //,bspline
-    AddSwitch("","adjust","adjust sky level");
-    AddSwitch("","expomask","use exposure mask");
-    AddSwitch("","average","compute average instead of summation");
-    COMMON_OPTIONS_BITPIX
+    AddOption("i","interpol","[near,bilinear,bicubic,bi3conv] interpol");
+    AddSwitch("","disable-back","does not subtract background");
+    AddSwitch("","arith","compute average by arithmetical mean");
     COMMON_OPTIONS_SINGLE
-    AddParam("file(s)",wxCMD_LINE_VAL_STRING,wxCMD_LINE_PARAM_MULTIPLE);
+    COMMON_OPTIONS_BITPIX
+    AddParam("file(s)",wxCMD_LINE_VAL_STRING,
+	     wxCMD_LINE_PARAM_OPTIONAL|wxCMD_LINE_PARAM_MULTIPLE);
   }
 
   else if( action == "phcal" ) {
@@ -529,6 +558,7 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
     AddSwitch(wxEmptyString,"list","show available photometry systems");
     AddOption(wxEmptyString,"tol","search radius [deg]",wxCMD_LINE_VAL_DOUBLE);
     AddOption("","saper","select aperture number",wxCMD_LINE_VAL_NUMBER);
+    COMMON_OPTIONS_CLOBBER
     COMMON_OPTIONS_MULTIPLE
     AddParam("file(s)[,results(s)]",wxCMD_LINE_VAL_STRING,
   	     wxCMD_LINE_PARAM_MULTIPLE|wxCMD_LINE_PARAM_OPTIONAL);
@@ -631,7 +661,6 @@ MuniCmdLineParser::MuniCmdLineParser(const wxString& action, int xargc, wchar_t
     COMMON_OPTIONS_BITPIX
   }
 
-
 }
 
 MuniCmdLineParser::~MuniCmdLineParser()
@@ -659,3 +688,66 @@ void MuniCmdLineParser::Usage() const
     wxPrintf(GetUsageString());
 
 }
+
+bool MuniCmdLineParser::InputByFile(const wxString& lstname)
+{
+  wxASSERT(files.IsEmpty());
+
+  wxFile lstfile;
+  if( lstname.IsEmpty() )
+    lstfile.Attach(wxFile::fd_stdin);
+  else
+    lstfile.Open(lstname);
+
+  if( lstfile.IsOpened() ) {
+    wxFileInputStream input(lstfile);
+    wxTextInputStream text(input);
+    while( input.IsOk() && input.CanRead() ) {
+      wxString line(text.ReadLine());
+      wxString filename = line.BeforeFirst('#');
+      filename.Trim();
+      if( ! filename.IsEmpty() )
+	files.push_back(filename);
+    }
+  }
+
+  if( lstname.IsEmpty() )
+    // Stdin must be pulled back to the parent process,
+    // else sockets of calling process are unavailable.
+    lstfile.Detach();
+
+  return ! files.IsEmpty();
+}
+
+wxString MuniCmdLineParser::GetFile(size_t n) const
+{
+  wxASSERT(n < files.size());
+  return files[n];
+}
+
+wxArrayString MuniCmdLineParser::GetFiles(size_t n) const
+{
+  wxASSERT(n < files.size());
+
+  wxArrayString filenames;
+  wxStringTokenizer tokenizer(files[n]);
+  while ( tokenizer.HasMoreTokens() ) {
+    wxString file = tokenizer.GetNextToken();
+    filenames.push_back(file);
+  }
+
+  return filenames;
+}
+
+int MuniCmdLineParser::Parse(bool giveUsage)
+{
+  int result = wxCmdLineParser::Parse(giveUsage);
+
+  if( result != 0 )
+    return result;
+
+  for(size_t i = 0; i < GetParamCount(); i++)
+    files.push_back(GetParam(i));
+
+  return result;
+}
