@@ -2,7 +2,7 @@
 
   FITS manipulations
 
-  Copyright © 2011-7 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2011-8 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -32,7 +32,6 @@
 #include "munipack.h"
 #include <wx/wx.h>
 #include <wx/tokenzr.h>
-#include <wx/filename.h>
 
 
 using namespace std;
@@ -43,7 +42,7 @@ bool Munipack::fits(MuniProcess *action, MuniCmdLineParser& cmd)
   wxString mode = "STRUCTURE";
   wxString a;
 
-  if( cmd.Found("K",&a) || cmd.Found("print-keys",&a) ) {
+  if( cmd.Found("K",&a) || cmd.Found("keys",&a) ) {
 
     mode = "HEADER";
 
@@ -61,22 +60,27 @@ bool Munipack::fits(MuniProcess *action, MuniCmdLineParser& cmd)
       action->Write("REMKEY = '" + tokenizer.GetNextToken() + "'");
   }
 
-  else if( cmd.Found("update-key",&a) ) {
+  else if( cmd.Found("update") ) {
 
     mode = "HEADER";
 
-    wxString line("UPDATEKEY =");
-    wxStringTokenizer tokenizer(a, ",=");
-    for(size_t i = 0; i < 3; i++) {
-      if( tokenizer.HasMoreTokens() ) {
-	wxString s(tokenizer.GetNextToken());
-	s.Replace("'","''");
-	line += " '"+s+"'";
-      }
-      else
-	line += " ''";
+    action->Write("UPDATE = T");
+
+    if( cmd.Found("key",&a) )
+      action->Write("KEY = '"+a+"'");
+
+    if( cmd.Found("val",&a) ) {
+      wxString b = a.Find("'") == wxNOT_FOUND ? "'" : "\"";
+      action->Write("VALUE = "+b+a+b);
     }
-    action->Write(line);
+
+    if( cmd.Found("com",&a) ) {
+      wxString b = a.Find("'") == wxNOT_FOUND ? "'" : "\"";
+      action->Write("COMMENT = "+b+a+b);
+    }
+
+    if( cmd.Found("templ",&a) )
+      action->Write("TEMPL = '"+a+"'");
   }
 
   else if( cmd.Found("lh") || cmd.Found("header") ) {
@@ -106,6 +110,11 @@ bool Munipack::fits(MuniProcess *action, MuniCmdLineParser& cmd)
 
     mode = "RESTORE";
 
+  }
+  else if( cmd.Found("cat") ) {
+
+    mode = "CAT";
+
   }
   else if( cmd.Found("remove-extensions",&a) ) {
 
@@ -127,7 +136,7 @@ bool Munipack::fits(MuniProcess *action, MuniCmdLineParser& cmd)
   CommonOutputMultiple(action,cmd);
 
   if( ! cmd.Found("O") && (mode == "RESTORE" || mode == "DUMP") ) {
-    // In this modes, .lst are defaultly used
+    // In this modes, .lst are by default
 
     SetAdvanced(true);
     if( mode == "RESTORE" ) {
@@ -140,8 +149,7 @@ bool Munipack::fits(MuniProcess *action, MuniCmdLineParser& cmd)
     }
   }
 
-  for(size_t i = 0; i < cmd.GetParamCount(); i++)
-    WriteFiles(action,cmd.GetParam(i));
+  WriteFiles(action,cmd);
 
   return true;
 }
