@@ -3,7 +3,7 @@
   Munipack
 
 
-  Copyright © 2009-2016 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2009-2018 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -29,7 +29,6 @@
 #include <wx/cmdline.h>
 #include <wx/process.h>
 #include <wx/event.h>
-#include <queue>
 
 
 class MuniCmdLineParser: public wxCmdLineParser
@@ -39,11 +38,18 @@ public:
   virtual ~MuniCmdLineParser();
 
   void Usage() const;
+  int Parse(bool giveUsage=true);
+  size_t GetFilesCount() const { return files.size(); }
+  wxString GetFile(size_t n) const;
+  wxArrayString GetFiles(size_t n) const;
+  wxArrayString GetFiles() const { return files; }
 
 private:
 
   int argc;
   wchar_t **argv;
+  wxArrayString files;
+  bool InputByFile(const wxString&);
 
 };
 
@@ -60,18 +66,22 @@ private:
 
   void CommonOptionsBitpix(MuniProcess *, const MuniCmdLineParser&);
   void CommonOptionsPhCorr(MuniProcess *, const MuniCmdLineParser&);
-  void CommonOptionsArith(MuniProcess *, const MuniCmdLineParser&);
+  void CommonOptionsClobber(MuniProcess *, const MuniCmdLineParser&);
   void CommonOptionsPhq(MuniProcess *, const MuniCmdLineParser&);
-  void CommonOptionsAddKey(MuniProcess *, const MuniCmdLineParser&);
   void CommonOutputMultiple(MuniProcess *, const MuniCmdLineParser&);
-  void CommonOutputSingle(MuniProcess *, const MuniCmdLineParser&, const wxString& =wxEmptyString);
+  void CommonOutputSingle(MuniProcess *, const MuniCmdLineParser&,
+			  const wxString& =wxEmptyString);
+  void EnvironmentPhCorr(MuniProcess *);
+  void WriteOutput(MuniProcess *, const wxString&) const;
+  void WriteFiles(MuniProcess *, const MuniCmdLineParser&) const;
+  void WriteFiles(MuniProcess *, const wxArrayString&) const;
+  void WriteFile(MuniProcess *, const wxString&) const;
 
   bool bias(MuniProcess *,MuniCmdLineParser&);
   bool dark(MuniProcess *,MuniCmdLineParser&);
   bool flat(MuniProcess *,MuniCmdLineParser&);
   bool phcorr(MuniProcess *,MuniCmdLineParser&);
-  bool coloring(MuniProcess *,MuniCmdLineParser&);
-  bool ctrafo(MuniProcess *,MuniCmdLineParser&);
+  bool colouring(MuniProcess *,MuniCmdLineParser&);
   bool cone(MuniProcess *,MuniCmdLineParser&);
   bool astrometry(MuniProcess *,MuniCmdLineParser&);
   bool phcal(MuniProcess *,MuniCmdLineParser&);
@@ -92,29 +102,21 @@ private:
 
   void cone_lists(const MuniCmdLineParser&);
 
-  void WriteFile(MuniProcess *, const wxString&);
-  void WriteFiles(MuniProcess *, const wxString&);
-  void WriteOutput(MuniProcess *, const wxString&);
-  bool InputBy(MuniProcess *, const wxString&,bool);
   void apstr(const wxString&, long&, wxString&);
+  void dblstr(const wxString&, const wxString&, long&, wxString&);
 
-  void GetResult(const wxString&, wxString&, wxString&, wxString&);
-  long FindBackup(const wxString&) const;
-  wxString GetBackup(const wxString&) const;
+  wxString GetAdvanced(const wxString&) const;
   wxString GetOutput(const wxString&) const;
+  wxString GetFormat(size_t) const;
 
   void SetAdvanced(bool b) { advanced = b; }
   void SetPattern(const wxString& a) { pattern = a; }
   void SetMask(const wxString& a) { mask = a; }
   void SetFormat(const wxString& a) { format = a; }
-  void SetSuffix(const wxString& a) { suffix = a; }
-  void SetControl(const wxString& a) { control = a; }
   bool SetTargetDir(const wxString&);
-  void EnableBackup(bool e =true) { control= e ? "existing" : "none"; }
 
   bool advanced;
-  wxString pattern, mask, format, suffix, control, target;
-  int ncount;
+  wxString pattern, mask, format, target;
 
   MuniPipe pipe;
 };
