@@ -2,7 +2,7 @@
 
   Photometrical calibration
 
-  Copyright © 2012-7 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2012-8 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -24,21 +24,21 @@
 #include "munipack.h"
 #include "vocatconf.h"
 #include <wx/wx.h>
-#include <wx/log.h>
-#include <wx/tokenzr.h>
 #include <wx/filename.h>
 
 
 bool Munipack::phcal(MuniProcess *action, MuniCmdLineParser& cmd)
 {
   CommonOutputMultiple(action,cmd);
-
-  if( ! cmd.Found("O") ) {
-    SetAdvanced(true);
-    SetMask("!\\1_cal.\\2");
-  }
+  CommonOptionsClobber(action,cmd);
 
   wxString a;
+  if( !  cmd.Found("O") &&
+      ! (cmd.Found("target-directory",&a) || cmd.Found("t",&a)) ) {
+    SetAdvanced(true);
+    SetMask("\\1_cal.\\2");
+  }
+
   if( cmd.Found("phsystab",&a) )
     action->Write("PHSYSTABLE = '" + a + "'");
 #ifdef MUNIPACK_DATA_DIR
@@ -144,9 +144,7 @@ bool Munipack::phcal(MuniProcess *action, MuniCmdLineParser& cmd)
       action->Write("CAT = '" + c + "'");
     }
 
-    action->Write("NFILE = %d",cmd.GetParamCount());
-    for(size_t i = 0; i < cmd.GetParamCount(); i++)
-      WriteFiles(action,cmd.GetParam(i));
+    WriteFiles(action,cmd);
   }
 
   return true;
