@@ -990,7 +990,6 @@ void MuniAstrometry::RunProcessing()
   wxASSERT(astropt);
 
   MuniProcess *astrometry = new MuniProcess(&pipe,"astrometry");
-  astrometry->SetVerbose(false);
   pipe.push(astrometry);
 
   astrometry->Write("PIPELOG = T");
@@ -1030,7 +1029,8 @@ void MuniAstrometry::RunProcessing()
     else
       astrometry->Write("REF = '"+reffile+"'");
   }
-  astrometry->WriteFiles(workingfile,wxEmptyString,wxEmptyString);
+
+  astrometry->Write("FILE = '"+workingfile+"' ''");
 }
 
 void MuniAstrometry::OnCalibrate(wxCommandEvent& event)
@@ -1415,7 +1415,7 @@ void MuniAstrometry::OnSaveWCS(wxCommandEvent& event)
   action->Write(a);
   action->Write("SCALE = %25.15e",scale);
   action->Write("ANGLE = %20.15f",angle);
-  action->WriteFile(workingfile);
+  action->Write("FILE = '"+workingfile+"' ''");
 
   pipe.Start();
 }
@@ -1448,7 +1448,7 @@ void MuniAstrometry::OnRemove(wxCommandEvent& event)
   pipe.push(action);
 
   action->Write("WCSSAVE = T");
-  action->WriteFile(workingfile);
+  action->Write("FILE= '"+workingfile+"' ''");
 
   pipe.Start();
 }
