@@ -2,7 +2,7 @@
 
   xmunipack - display canvas
 
-  Copyright © 2012-14, 17 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2012-14, 2018-9 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -29,6 +29,7 @@
 #include <wx/graphics.h>
 #include <wx/renderer.h>
 #include <wx/thread.h>
+#include <wx/clipbrd.h>
 #include <list>
 
 using namespace std;
@@ -39,38 +40,36 @@ using namespace std;
 // -- MuniDisplayCanvas
 
 MuniDisplayCanvas::MuniDisplayCanvas(wxWindow *w, MuniConfig *c):
-  wxScrolledCanvas(w,wxID_ANY),render(0),topwin(GetGrandParent()),config(c),handler(0),
-  xoff(0), yoff(0),
-  display(config->display_colorspace),
-  zoom(config->display_zoom),zoomfit(-1.0),zooming(true),dragging(false),rendering(false),fit(false),astrometry(false)
+  wxScrolledCanvas(w,wxID_ANY),render(0),canvasid(0),
+  topwin(GetGrandParent()),config(c),handler(0),
+  xoff(0), yoff(0), value_type(UNIT_COUNT), coo_type(COO_PIXEL),
+  scaleicon(false), shrink(1),zoom(1),palauto(false),
+  zooming(false), shrinking(false),tunning(false), completed(false),
+  dragging(false),rendering(false), astrometry(false)
 {
   wxLogDebug("MuniDisplayCanvas constructor..");
 
   SetBackgroundStyle(wxBG_STYLE_PAINT);
 
   pal.SetPalette(config->display_pal);
-  itt.SetItt(config->display_itt);
-  display.SetTemperature(config->color_temperature);
-  display.SetColorspace(config->display_colorspace);
-  color.SetTrans("XYZ",config->cdatafile);
-  color.SetWhitePoint(config->color_uwhitepoint,config->color_vwhitepoint);
+  pal.SetInverse(config->display_palinv);
 
   Bind(wxEVT_CLOSE_WINDOW,&MuniDisplayCanvas::OnClose,this);
   Bind(wxEVT_SIZE,&MuniDisplayCanvas::OnSize,this);
-  Bind(wxEVT_SCROLLWIN_TOP,&MuniDisplayCanvas::OnScroll,this);
-  Bind(wxEVT_SCROLLWIN_BOTTOM,&MuniDisplayCanvas::OnScroll,this);
-  Bind(wxEVT_SCROLLWIN_LINEUP,&MuniDisplayCanvas::OnScroll,this);
-  Bind(wxEVT_SCROLLWIN_PAGEUP,&MuniDisplayCanvas::OnScroll,this);
-  Bind(wxEVT_SCROLLWIN_LINEDOWN,&MuniDisplayCanvas::OnScroll,this);
-  Bind(wxEVT_SCROLLWIN_PAGEDOWN,&MuniDisplayCanvas::OnScroll,this);
-  Bind(wxEVT_SCROLLWIN_THUMBRELEASE,&MuniDisplayCanvas::OnScroll,this);
   Bind(wxEVT_PAINT,&MuniDisplayCanvas::OnPaint,this);
   Bind(wxEVT_IDLE,&MuniDisplayCanvas::OnIdle,this);
-  Bind(EVT_RENDER,&MuniDisplayCanvas::OnRenderFinish,this,ID_RENDER);
+  Bind(EVT_RENDER,&MuniDisplayCanvas::OnRenderFinish,this);
   Bind(EVT_RENDER,&MuniDisplayCanvas::OnSubRender,this,ID_SUBRENDER);
-  Bind(EVT_CONFIG_UPDATED,&MuniDisplayCanvas::OnConfigUpdated,this);
   Bind(wxEVT_KEY_DOWN,&MuniDisplayCanvas::OnKeyDown,this);
   Bind(wxEVT_RIGHT_DOWN,&MuniDisplayCanvas::OnMenu,this);
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniDisplayCanvas::OnClipValue,this,
+       ID_CLIP_VALUE);
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniDisplayCanvas::OnClipCoo,this,
+       ID_CLIP_COO);
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniDisplayCanvas::OnValueType,this,
+       ID_VALTYPE);
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniDisplayCanvas::OnCooType,this,
+       ID_COOTYPE);
 }
 
 MuniDisplayCanvas::~MuniDisplayCanvas()
@@ -81,10 +80,10 @@ MuniDisplayCanvas::~MuniDisplayCanvas()
     delete *i;
   */
 
-  config->display_itt = itt.GetItt();
-  config->display_pal = pal.GetPalette();
-  config->display_zoom = zoom;
-
+  if( ! palauto ) {
+    config->display_pal = pal.GetPalette();
+    config->display_palinv = pal.GetInverse();
+  }
 }
 
 void MuniDisplayCanvas::OnClose(wxCloseEvent& event)
@@ -100,70 +99,62 @@ void MuniDisplayCanvas::OnClose(wxCloseEvent& event)
     event.Skip();
 }
 
-void MuniDisplayCanvas::OnScroll(wxScrollWinEvent& event)
-{
-  rendering = true;
-  event.Skip();
-}
-
 void MuniDisplayCanvas::OnIdle(wxIdleEvent& event)
 {
+  //  wxLogDebug("MuniDisplayCanvas::OnIdle");
+
   // Rendering is started by Idle event FOLLOWING all (resize) events
   // The right canvas size is prepared for drawing.
 
   if( rendering ) {
-
     if( render ) {
       StopRendering();
       event.RequestMore();
     }
     else {
       rendering = false;
-      Render();
+      StartRendering();
     }
   }
+
 }
 
 void MuniDisplayCanvas::OnSize(wxSizeEvent& event)
 {
-  //  wxLogDebug("MuniDisplayCanvas::OnSize %d %d",
-  //	     event.GetSize().GetWidth(),event.GetSize().GetHeight());
-
-  if( canvas.IsOk() ) {
-    wxSize s(GetSize());
-    xoff = wxMax((s.GetWidth() - canvas.GetWidth())/2,0);
-    yoff = wxMax((s.GetHeight() - canvas.GetHeight())/2,0);
-    zoomfit = GetFitZoom();
-  }
-
-  Refresh();
-  event.Skip();
+  UpdateScroll();
 }
 
 void MuniDisplayCanvas::OnPaint(wxPaintEvent& event)
 {
-  //  wxLogDebug("OnPaint");
   wxPaintDC dc(this);
+  DoPrepareDC(dc);
 
-  /*
-  dc.SetAxisOrientation(true,true);
-  dc.SetDeviceOrigin(xoff,yoff);
-  */
+  if( zoom > 1 )
+    dc.SetUserScale(double(zoom),double(zoom));
 
-  dc.SetBackground(*wxBLACK_BRUSH);
+  if( pal.GetInverse() )
+    dc.SetBackground(*wxWHITE_BRUSH);
+  else
+    dc.SetBackground(*wxBLACK_BRUSH);
 
-  if( xoff > 0 || yoff > 0 ) // faster ?
-    dc.Clear();
+  dc.Clear();
+
+  if( canvas.IsOk() )
+    dc.DrawBitmap(wxBitmap(canvas),xoff,yoff);
+
+  // Layers
+  list<MuniLayer>::const_iterator i;
+  for(i = layers.begin(); i != layers.end(); ++i)
+    DrawLayer(dc,*i);
+
+  return;
+
+  // it's kept for later development
 
   int x0,y0,wc,hc;
   GetViewStart(&x0,&y0);
   GetClientSize(&wc,&hc);
 
-  double z = 1.0;
-  if( hdu.IsOk() && icon.IsOk() ) {
-    z = (hdu.Width()*zoom)/icon.GetWidth();
-  }
-
   wxRegionIterator upd = GetUpdateRegion();
   while(upd) {
     wxRect r = upd.GetRect();
@@ -172,35 +163,14 @@ void MuniDisplayCanvas::OnPaint(wxPaintEvent& event)
     int ys = r.GetY();
     int ws = r.GetWidth();
     int hs = r.GetHeight();
-    //    wxLogDebug("demaged: %d %d %d %d",xs,ys,ws,hs);
+    wxLogDebug("damaged: %d %d %d %d",xs,ys,ws,hs);
 
     int xw = xoff > 0 ? xs - xoff : x0 + xs;
     int yw = yoff > 0 ? ys - yoff : y0 + ys;
     int ww = ws - 2*xoff;
     int hw = hs - 2*yoff;
 
-    if( icon.IsOk() ) {
-
-      int xb = wxMax(xw/z,0);
-      int yb = wxMax(yw/z,0);
-      int wb = wxMin(ww/z + 1,icon.GetWidth());
-      int hb = wxMin(hw/z + 1,icon.GetHeight());
-
-      bool wt = 0 < wb && wb <= icon.GetWidth();
-      bool ht = 0 < hb && hb <= icon.GetHeight();
-      bool xt = xb >= 0 && xb + wb <= icon.GetWidth();
-      bool yt = yb >= 0 && yb + hb <= icon.GetHeight();
-
-      if( wt && ht && xt && yt) {
-	wxImage i(icon.GetSubImage(wxRect(xb,yb,wb,hb)));
-	i.Rescale(wb*z,hb*z);
-	dc.DrawBitmap(wxBitmap(i),xs+xoff,ys+yoff);
-      }
-      else
-	wxLogDebug("failed icon: %d %d %d %d",xb,yb,wb,hb);
-    }
-
-    if( canvas.IsOk() ) {
+    if( false && canvas.IsOk() ) {
 
       int xb = wxMax(xw,0);
       int yb = wxMax(yw,0);
@@ -219,278 +189,193 @@ void MuniDisplayCanvas::OnPaint(wxPaintEvent& event)
       if( wt && ht && xt && yt ) {
 
 	wxImage i(canvas.GetSubImage(wxRect(xb,yb,wb,hb)));
-	if( zoom > 1.0 ) i.Rescale(zoom*wb,zoom*hb);
+	if( zoom > 1 ) {
+	  i.Rescale(zoom*wb,zoom*hb);
+	}
 	dc.DrawBitmap(wxBitmap(i),xs+xoff,ys+yoff);
       }
       else
 	wxLogDebug("failed canvas: %d %d %d %d",xb,yb,wb,hb);
-
     }
 
     upd++;
   }
 
-  // Layers
-  list<MuniLayer>::const_iterator i;
-  for(i = layers.begin(); i != layers.end(); ++i)
-    DrawLayer(dc,*i);
-
 }
 
+
 void MuniDisplayCanvas::SetHandler(wxEvtHandler *h)
 {
   handler = h;
 
   if( handler ) {
     Bind(wxEVT_MOTION,&MuniDisplayCanvas::OnMouseMotion,this);
-    Bind(wxEVT_MOUSEWHEEL,&MuniDisplayCanvas::OnMouseWheel,this);
+    //    Bind(wxEVT_MOUSEWHEEL,&MuniDisplayCanvas::OnMouseWheel,this);
     Bind(wxEVT_LEFT_UP,&MuniDisplayCanvas::OnLeftUp,this);
   }
   else {
     Unbind(wxEVT_MOTION,&MuniDisplayCanvas::OnMouseMotion,this);
-    Unbind(wxEVT_MOUSEWHEEL,&MuniDisplayCanvas::OnMouseWheel,this);
+    //    Unbind(wxEVT_MOUSEWHEEL,&MuniDisplayCanvas::OnMouseWheel,this);
     Unbind(wxEVT_LEFT_UP,&MuniDisplayCanvas::OnLeftUp,this);
   }
 }
 
-
-void MuniDisplayCanvas::InitITT(const FitsArray& a, const FitsArrayStat& stat)
+void MuniDisplayCanvas::SetHdu(const FitsArray& a, const wxImage& i, bool pauto)
 {
-  if( a.IsColor() ) {
-    itt.SetKind(ITT_KIND_ABS);
-    itt.Init(FitsArrayStat(a.Plane(1)));
-    color.Init(config->cdatafile,a);
-  }
-  else {
-    itt.SetKind(ITT_KIND_REL);
-    itt.Init(FitsArrayStat(a));
-  }
-
-  itt_orig = itt;
-}
-
-void MuniDisplayCanvas::SetHdu(const FitsArray& a, const wxImage& i, const FitsArrayStat& stat)
-{
-  hdu = a;
+  array = a;
   icon = i;
+  if( icon.HasAlpha() )
+    icon.ClearAlpha();
+  wxASSERT(array.IsOk() && icon.IsOk());
 
-  wxASSERT(hdu.IsOk() && icon.IsOk());
-
-  Reset();
-
-  InitITT(a,stat);
-
-  fit = config->display_zoom <= 0.0;
-  SetInitZoom(config->display_zoom ? -1.0 : 1.0);
-
-  /*
-  int w = wxMax(int(zoom*hdu.Width()),1);
-  int h = wxMax(int(zoom*hdu.Height()),1);
-  */
-
-  InitDC(hdu.Width(),hdu.Height(),zoom);
-
-  SetScrollRate(1,1);
-
-  fitsimage = FitsImage(hdu);
+  fitsimage = FitsImage(array);
   wxASSERT(fitsimage.IsOk());
 
-  fimage = fitsimage;
+  FitsArray stat(array.IsColor() ? array.Plane(1) : array);
+  tone.Setup(stat.Npixels(),stat.PixelData());
 
-  zoomfit = GetFitZoom();
+  palauto = pauto;
+  if( palauto )
+    SelectAutoPal();
+
+  // Init canvas
+  InitCanvas();
+  rendering = true;
+  shrinking = true;
+  tunning = true;
+
+  // setup zoom
+  MuniFullTuneEvent t(EVT_FULLTUNE);
+  t.tone = tone;
+  t.itt = itt;
+  t.pal = pal;
+  t.colour = color;
+  wxQueueEvent(handler,t.Clone());
 
   Refresh();
-  rendering = true;
-
-  /*
-  MuniLayer l;
-  l.objects.push_back(new MuniDrawCircle(382.5,255.0,382.5));
-  layers.push_back(l);
-  */
 }
 
-void MuniDisplayCanvas::InitDC(int w0, int h0, double z)
+void MuniDisplayCanvas::InitCanvas()
 {
-  //  wxLogDebug("MuniDisplayCanvas::InitDC %d %d %f",w0,h0,z);
+  canvasid = 007;
+  shrink = GetFitShrink();
+  zoom = 1;
+  canvas = icon.Scale(fitsimage.GetWidth()/shrink,fitsimage.GetHeight()/shrink);
+  SetVirtualSize(canvas.GetWidth(),canvas.GetHeight());
+  UpdateScroll();
 
-  int wc,hc;
-  GetClientSize(&wc,&hc);
-
-  int w = z*w0;
-  int h = z*h0;
-
-  int wv = wxMax(w,wc);
-  int hv = wxMax(h,hc);
-
-  SetVirtualSize(wv,hv);
-
-  if( z > 1.0 )
-    canvas = wxImage(w0,h0);
-  else
-    canvas = wxImage(w,h);
-
-  xoff = wxMax((wc - canvas.GetWidth())/2,0);
-  yoff = wxMax((hc - canvas.GetHeight())/2,0);
-  //  wxLogDebug("%d %d %d %d %f",w,h,xoff,yoff,z);
-
-  canvas.InitAlpha();
-  for(int i = 0; i < canvas.GetWidth(); i++)
-    for(int j = 0; j < canvas.GetHeight(); j++)
-      canvas.SetAlpha(i,j,wxIMAGE_ALPHA_TRANSPARENT);
+  MuniTuneEvent ev(EVT_TUNE,ID_ZOOM);
+  ev.n = shrink;
+  ev.shrink = true;
+  wxQueueEvent(topwin,ev.Clone());
 }
 
+void MuniDisplayCanvas::UpdateCanvas(int shrink, int zoom)
+{
+  if( canvas.IsOk() ) {
+    int w = fitsimage.GetWidth();
+    int h = fitsimage.GetHeight();
+    if( shrink >= 1 )
+      canvas.Rescale(w/shrink,h/shrink);
+  }
+  SetVirtualSize(zoom*canvas.GetWidth(),zoom*canvas.GetHeight());
 
+  canvasid = canvasid + 1;
+}
+
+void MuniDisplayCanvas::UpdateCanvas()
+{
+  canvasid = canvasid + 1;
+}
 
 
 void MuniDisplayCanvas::Reset()
 {
-  itt.SetItt(config->display_itt);
-  itt = itt_orig;
+  StopRendering();
+
+  itt.Reset();
+  pal.Reset();
+  color.Reset();
+  tone.Reset();
   pal.SetPalette(config->display_pal);
-  pal.SetNegative(false);
-  color.SetSaturation(1.0);
-  color.SetHue(0.0);
-  color.SetNightVision(config->display_nvision);
-  color.SetWhitePoint(config->color_uwhitepoint,config->color_vwhitepoint);
-  color.SetNightThresh(0.0);
-  color.SetNightWidth(30.0); // must correspods to initializer of Color?
-
-  SetInitZoom(config->display_zoom ? -1.0 : 1.0);
 
+  InitCanvas();
   rendering = true;
+  shrinking = true;
+  tunning = true;
 }
 
-void MuniDisplayCanvas::Render()
+void MuniDisplayCanvas::StartRendering()
 {
-  //  wxASSERT(!wxIsBusy() && fitsimage.IsOk());
-  //  wxLogDebug("MuniDisplayCanvas::Render");
-
-  //stop when we are already rendering
-  //StopRendering(); ?????
-
-  // render image
-  display.SetItt(itt);
-  display.SetPalette(pal);
-  display.SetColor(color);
-
-  /*
-  const vector<FitsArray> band = fitsimage.GetArrays();
-  for(size_t i = 0; i < band.size(); i++) {
-    if( band[i].IsOk() )
-      wxLogDebug("%f %f %f",band[i].Pixel(1206,1276),band[i].Pixel(1211,1284),band[i].Pixel(911,614));
-  }
-  */
-
-  int x0,y0,wc,hc,ws,hs;
-  GetViewStart(&x0,&y0);
-  GetClientSize(&wc,&hc);
-  GetSize(&ws,&hs);
-
-  double z = zoom > 1.0 ? 1.0 : zoom;
-  int x = wxMax(int((x0 - xoff)/z),0);
-  int y = wxMax(int((y0 - yoff)/z),0);
-  int w = wxMin(int(wc/z),fitsimage.GetWidth());
-  int h = wxMin(int(hc/z),fitsimage.GetHeight());
-
-  if( ws > wc ) {
-    x = x0;
-    w = wc;
-  }
-  else {
-    x = 0;
-    w = wxMin(ws,fitsimage.GetWidth());
-  }
-
-  if( hs > hc ) {
-    y = hc - y0;
-    h = hc;
-  }
-  else {
-    h = wxMin(hs,fitsimage.GetHeight());
-    y = 0;
-  }
-
-  x = 0;
-  y = 0;
-  w = fitsimage.GetWidth();
-  h = fitsimage.GetHeight();
-
-  wxASSERT(zoom > 0.0);
-
-  if( xoff > 0 )
-    x = 0;
-  else
-    x = wxMax(x0,0)/zoom;
-
-  if( yoff > 0 )
-    y = 0;
-  else
-    y = wxMax(fitsimage.GetHeight() - (hs + y0)/zoom,0);
-
-  w = wxMin(fitsimage.GetWidth(),ws/zoom);
-  h = wxMin(fitsimage.GetHeight(),hs/zoom);
-
-  //  wxLogDebug("%f %d %d %d %d %d %d %d %d %d %d",zoom,x,y,w,h,x0,y0,wc,hc,xoff,yoff);
-
   wxASSERT(render == 0);
-  render = new ImageRender(this,fitsimage,fimage,display,zoom,zooming,x,y,w,h);
-  wxASSERT(render);
-  wxThreadError code = render->Create();
-  wxASSERT(code == wxTHREAD_NO_ERROR);
-  render->Run();
 
-  //  ::wxBeginBusyCursor();
-  Refresh();
+  ::wxBeginBusyCursor();
+
+  if( shrinking ) {
+    completed = false;
+    render = new DisplayShrinkRender(this,fitsimage,tone,shrink,canvasid);
+    wxThreadError code = render->Create();
+    wxASSERT(code == wxTHREAD_NO_ERROR);
+    render->Run();
+    shrinking = false;
+  }
+  else if( tunning ) {
+    wxASSERT(scaled.IsOk());
+    render = new DisplayTuneRender(this,scaled,tone,itt,pal,color,canvasid);
+    wxThreadError code = render->Create();
+    wxASSERT(code == wxTHREAD_NO_ERROR);
+    render->Run();
+    tunning = false;
+  }
 }
 
 
 void MuniDisplayCanvas::OnSubRender(MuniRenderEvent& event)
 {
-  //  wxLogDebug("MuniDisplay::OnSubRender %d %d %d %d %d",event.x,event.y,event.width,event.height,(int)renderid);
+  if( event.id != canvasid || ! event.picture.IsOk() )
+    return;
 
-  picture = event.picture;
-  wxASSERT(picture.IsOk());
+  wxImage img(event.picture.GetWidth(),event.picture.GetHeight(),
+  	      event.picture.NewTopsyTurvyRGB());
 
-  wxImage img(picture.GetWidth(),picture.GetHeight(),picture.NewTopsyTurvyRGB());
+  if( img.IsOk() && canvas.IsOk() ) {
+    /*
+    wxLogDebug("MuniDisplayCanvas::OnSubRender %d %d %d %d %d",
+	       (int)event.x,(int)event.y,
+	       (int)event.x + img.GetWidth(),
+	       (int)event.y + img.GetHeight(),int(ren));
+    */
+    wxASSERT(event.x >= 0 &&  event.y >= 0 &&
+	     event.x + img.GetWidth() <= canvas.GetWidth() &&
+	     event.y + img.GetHeight() <= canvas.GetHeight() );
+    canvas.Paste(img,event.x,event.y);
+  }
 
-  int x0,y0;
-  GetViewStart(&x0,&y0);
-
-  wxASSERT(canvas.IsOk() && canvas.HasAlpha());
-  for(int i = 0; i < event.width; i++)
-    for(int j = 0; j < event.height; j++) {
-      int k = event.x + x0 + i;
-      int l = event.y + y0 + j;
-      if( 0 <= k && k < canvas.GetWidth() && 0 <= l && l < canvas.GetHeight() ) {
-	canvas.SetRGB(wxRect(k,l,1,1),img.GetRed(i,j),img.GetGreen(i,j),img.GetBlue(i,j));
-	canvas.SetAlpha(k,l,wxIMAGE_ALPHA_OPAQUE);
-      }
-    }
   Refresh();
 }
 
 void MuniDisplayCanvas::OnRenderFinish(MuniRenderEvent& event)
 {
-  //  wxLogDebug("MuniDisplay::OnRenderFinish");
+  wxLogDebug("MuniDisplayCanvas::OnRenderFinish %d ",
+	     event.GetId()==ID_RENDER_SHRINK);
 
-  fimage = event.image;
-  zooming = false;
+  if( event.image.IsOk() && event.GetId() == ID_RENDER_SHRINK &&
+      event.completed ) {
+    completed = event.completed;
+    scaled = event.image;
+  }
 
-  //  wxASSERT(render == 0);
-  //  render = 0;
+  if( event.GetId() == ID_RENDER_SHRINK && scaled.IsOk() ) {
+    rendering = true;
+    tunning = true;
+  }
 
-  // if ! fimage.IsOk() ?????
-  //  OverlayBitmap();
-
-  Refresh();
-
-  //  wxASSERT(::wxIsBusy()); // ??? failed randomly?
-  //  ::wxEndBusyCursor();
+  ::wxEndBusyCursor();
 }
 
 void MuniDisplayCanvas::StopRendering()
 {
-  //  wxLogDebug("MuniDisplayCanvas::StopRendering()");
+  wxLogDebug("MuniDisplayCanvas::StopRendering()");
   {
     wxCriticalSectionLocker enter(renderCS);
     if( render )
@@ -504,22 +389,25 @@ void MuniDisplayCanvas::StopRendering()
     }
     ::wxMilliSleep(1);
   }
+
+  rendering = false;
 }
 
+
 void MuniDisplayCanvas::ConvertCoo(double u, double v, double& x, double& y)
 {
-  double height = canvas.GetHeight();
-  if( zoom > 1.0 ) height = zoom*height;
-
-  x = xoff + zoom*u;
-  y = yoff + height - zoom*v;
-  x = x - vbX;
-  y = y - vbY;
-
-  /*
-  x = u;
-  y = height - v;
-  */
+  if( zoom > 1 ) {
+    x = xoff + zoom*u;
+    y = yoff + (canvas.GetHeight() - v)*zoom;
+    x = x - vbX;
+    y = y - vbY;
+  }
+  else {
+    x = xoff + u / shrink;
+    y = yoff - v / shrink + canvas.GetHeight();
+    x = x - vbX;
+    y = y - vbY;
+  }
 }
 
 void MuniDisplayCanvas::DrawLayer(wxPaintDC& dc, const MuniLayer& layer)
@@ -533,7 +421,10 @@ void MuniDisplayCanvas::DrawLayer(wxPaintDC& dc, const MuniLayer& layer)
   // Find Out where the window is scrolled to
   //  vbX,vbY;                     // Top left corner of client
   GetViewStart(&vbX,&vbY);
-
+  /*
+  vbX = 0;
+  vbY = 0;
+  */
   //  wxLogDebug("%d",(int)layer.objects.size());
 
   //  wxColour blue(wxBLUE->Red(),wxBLUE->Green(),wxBLUE->Blue(),50);
@@ -795,41 +686,40 @@ void MuniDisplayCanvas::DrawLayer(wxPaintDC& dc, const MuniLayer& layer)
 
 // }
 
+wxPoint MuniDisplayCanvas::GetCartesianPosition(const wxPoint& point) const
+{
+  int height = fitsimage.GetHeight();
+
+  wxPoint cursor = point - wxPoint(xoff,yoff);
+
+  int x = cursor.x;
+  int y = cursor.y;
+  if( zoom > 1 ) {
+    x = x / zoom;
+    y = height - y / zoom;
+  }
+  else if( shrink >= 1 ) {
+    x = shrink * x;
+    y = height - shrink * y;
+  }
+
+  return wxPoint(x,y);
+}
+
 void MuniDisplayCanvas::OnMouseMotion(wxMouseEvent& event)
 {
-  if( astrometry )
-    return;
+  if( ! canvas.IsOk()) return;
+  if( astrometry ) return;
 
   wxClientDC dc(this);
   DoPrepareDC(dc);
 
   if( event.Moving() ) {
 
-    //    wxLogDebug(" %d %d",event.GetLogicalPosition(dc).x,event.GetLogicalPosition(dc).y);
-
-    wxPoint cursor = event.GetLogicalPosition(dc) - wxPoint(xoff,yoff);
     MuniSlewEvent evt(xEVT_SLEW);
-    evt.z_x = cursor.x;
-    evt.z_y = canvas.GetHeight() - cursor.y;
-    evt.i_x = cursor.x/zoom;
-    evt.i_y = (canvas.GetHeight() - cursor.y)/zoom;
-
-    //    wxLogDebug("%d %d %d %d",evt.i_x,evt.i_y,evt.z_x,evt.z_y);
-
-    int size = config->detail_zoom;
-
-    int x = evt.i_x;
-    int y = evt.i_y;
-
-    int x1 = x - size/2;
-    int y1 = y - size/2;
-    int x2 = x + size/2;
-    int y2 = y + size/2;
-
-    if( fitsimage.IsOk() &&
-	x1 > 0 && y1 > 0 && x2 < fitsimage.GetWidth() && y2 < fitsimage.GetHeight() ){
-      evt.picture = display.GetImage(fitsimage.GetSubImage(x1,y1,size,size));
-    }
+    crosshair = GetCartesianPosition(event.GetLogicalPosition(dc));
+    evt.x = crosshair.x;
+    evt.y = crosshair.y;
 
     wxASSERT(handler);
     wxQueueEvent(handler,evt.Clone());
@@ -850,11 +740,11 @@ void MuniDisplayCanvas::OnMouseMotion(wxMouseEvent& event)
       int x0,y0,dx,dy;
       GetViewStart(&x0,&y0);
       GetScrollPixelsPerUnit(&dx,&dy);
-      wxASSERT(dx > 0 && dy > 0);
-      int i = x0 - (event.GetLogicalPosition(dc).x - xdrag0) / dx;
-      int j = y0 - (event.GetLogicalPosition(dc).y - ydrag0) / dy;
-      Scroll(i,j);
-
+      if( dx > 0  && dy > 0 ) {
+	int i = x0 - (event.GetLogicalPosition(dc).x - xdrag0) / dx;
+	int j = y0 - (event.GetLogicalPosition(dc).y - ydrag0) / dy;
+	Scroll(i,j);
+      }
     }
 
   }
@@ -863,8 +753,10 @@ void MuniDisplayCanvas::OnMouseMotion(wxMouseEvent& event)
 
 void MuniDisplayCanvas::OnMouseWheel(wxMouseEvent& event)
 {
+  /*
   int id = event.GetWheelRotation() > 0 ? wxID_ZOOM_IN : wxID_ZOOM_OUT ;
   wxQueueEvent(topwin,new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED,id));
+  */
 }
 
 void MuniDisplayCanvas::OnLeftUp(wxMouseEvent& event)
@@ -872,16 +764,10 @@ void MuniDisplayCanvas::OnLeftUp(wxMouseEvent& event)
   if( dragging ) {
     SetCursor(*wxSTANDARD_CURSOR);
     dragging = false;
-    rendering = true;
   }
 }
 
-void MuniDisplayCanvas::OnStick(wxCommandEvent& event)
-{
-  stick = event.IsChecked();
-}
-
-void MuniDisplayCanvas::OnMenu(wxMouseEvent& WXUNUSED(event))
+void MuniDisplayCanvas::OnMenu(wxMouseEvent& event)
 {
   wxTopLevelWindow *twin = static_cast<wxTopLevelWindow *>(topwin);
   if( !twin ) return;
@@ -895,17 +781,68 @@ void MuniDisplayCanvas::OnMenu(wxMouseEvent& WXUNUSED(event))
     popup.Append(wxID_ZOOM_IN,"Zoom\tCtrl++");
     popup.Append(wxID_ZOOM_OUT,"Shrink\tCtrl+-");
     popup.Append(wxID_ZOOM_FIT,"Best fit\tCtrl+*");
-    popup.Append(ID_CYCLE_ITT,"Cycle Itt\tCtrl+T");
     popup.Append(ID_CYCLE_PAL,"Cycle Palette\tCtrl+P");
   }
   else /*if( ! twin->IsFullScreen() )*/ {
-    popup.AppendCheckItem(ID_STICK,"Stick"); // Ctrl+S does not works
-    popup.Check(ID_STICK,stick);
+    popup.Append(ID_CLIP_VALUE,"Copy value");
+    popup.Append(ID_CLIP_COO,"Copy coordinates");
   }
 
   PopupMenu(&popup);
 }
 
+void MuniDisplayCanvas::OnClipValue(wxCommandEvent& event)
+{
+  wxPoint p(crosshair);
+  FitsValue value(array,config->phsystemfile, config->fits_key_area,
+		    config->fits_key_exptime, config->fits_key_filter);
+  value.SetType(value_type);
+
+  wxString val;
+  if( array.IsColor() ) {
+    val =       "X = " + value.Get_str(p.x,p.y,2);
+    val = val + " Y = " + value.Get_str(p.x,p.y,1);
+    val = val + " Z = " + value.Get_str(p.x,p.y,0);
+  }
+  else
+    val = value.Get_str(p.x,p.y);
+
+  if(wxTheClipboard->Open() ) {
+    wxTheClipboard->SetData(new wxTextDataObject(val));
+    wxTheClipboard->Close();
+  }
+}
+
+void MuniDisplayCanvas::OnClipCoo(wxCommandEvent& event)
+{
+  wxPoint p(crosshair);
+  FitsCoo coords(array);
+  coords.SetType(coo_type);
+
+  wxString xcoo_str,ycoo_str;
+  if( coo_type == COO_PIXEL )
+    coords.GetPix(p.x,p.y,xcoo_str,ycoo_str);
+  else
+    coords.GetCoo(p.x,p.y,xcoo_str,ycoo_str);
+
+
+  if(wxTheClipboard->Open() ) {
+    wxTheClipboard->SetData(new wxTextDataObject(xcoo_str+" "+ycoo_str));
+    wxTheClipboard->Close();
+  }
+}
+
+void MuniDisplayCanvas::OnValueType(wxCommandEvent& event)
+{
+  wxASSERT(event.GetId() == ID_VALTYPE);
+  value_type = event.GetInt();
+}
+
+void MuniDisplayCanvas::OnCooType(wxCommandEvent& event)
+{
+  wxASSERT(event.GetId() == ID_COOTYPE);
+  coo_type = event.GetInt();
+}
 
 void MuniDisplayCanvas::OnKeyDown(wxKeyEvent& event)
 {
@@ -914,7 +851,8 @@ void MuniDisplayCanvas::OnKeyDown(wxKeyEvent& event)
   wxTopLevelWindow *twin = static_cast<wxTopLevelWindow *>(topwin);
 
   if( twin && twin->IsFullScreen() && keycode == WXK_ESCAPE ){
-    wxQueueEvent(topwin,new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED,ID_FULLSCREEN));
+    wxQueueEvent(topwin,new wxCommandEvent(wxEVT_COMMAND_MENU_SELECTED,
+					   ID_FULLSCREEN));
     return;
   }
 
@@ -926,8 +864,8 @@ void MuniDisplayCanvas::OnKeyDown(wxKeyEvent& event)
   if( event.GetModifiers() & wxMOD_CMD ) {
     int id = wxID_ANY;
     if( keycode == WXK_NUMPAD_MULTIPLY )
-      id = wxID_ZOOM_FIT;
-    else if( ! fit ) {
+      ;
+    else {
       switch(keycode) {
       case WXK_NUMPAD_ADD:      id = wxID_ZOOM_IN;  break;
       case WXK_NUMPAD_SUBTRACT: id = wxID_ZOOM_OUT; break;
@@ -962,91 +900,145 @@ void MuniDisplayCanvas::OnKeyDown(wxKeyEvent& event)
   event.Skip();
 }
 
-void MuniDisplayCanvas::OnTuneFine(MuniTuneEvent& e)
+void MuniDisplayCanvas::UpdateTune()
 {
-  //  wxLogDebug("MuniDisplayCanvas::OnTuneFine");
+  rendering = true;
+  tunning = true;
+  UpdateCanvas();
 
-  double oldzoom = zoom;
+  // inform zoom window on panel
+  MuniFullTuneEvent t(EVT_FULLTUNE);
+  t.tone = tone;
+  t.itt = itt;
+  t.pal = pal;
+  t.colour = color;
+  wxQueueEvent(handler,t.Clone());
+}
 
-  switch(e.GetId()) {
-  case ID_ITT_TYPE:   itt.SetItt(e.GetString()); break;
-  case ID_ITT_OFFSET: itt.SetBlack(e.x); break;
-    //  case ID_ITT_SLOPE:  itt.SetSensitivity(e.x); break;
-  case ID_ITT_BRIGHTNESS: itt.SetBrightness(e.x); break;
-  case ID_ITT_CONTRAST:  itt.SetContrast(e.x); break;
-  case ID_ITT_AMP:    itt.SetAmp(e.x); break;
-  case ID_ITT_ZERO:   itt.SetZero(e.x); break;
+void MuniDisplayCanvas::UpdateScale()
+{
+  rendering = true;
+  //  scaling = false;
+  shrinking = true;
+  tunning = true;
+  UpdateCanvas();
 
-  case ID_COLOR_NVISION: color.SetNightVision(e.GetInt()); break;
-  case ID_COLOR_SATUR:   color.SetSaturation(e.x); break;
-  case ID_COLOR_HUE:     color.SetHue(e.x); break;
-  case ID_COLOR_WHITEU:  color.SetWhitePoint(e.x,-1.0); break;
-  case ID_COLOR_WHITEV:  color.SetWhitePoint(-1.0,e.x); break;
-  case ID_COLOR_NTHRESH: color.SetNightThresh(e.x); break;
-  case ID_COLOR_NWIDTH:  color.SetNightWidth(e.x); break;
-  case ID_COLOR_MEAN:    color.SetLevel(e.index,e.x); break;
-  case ID_COLOR_WEIGHT:  color.SetWeight(e.index,e.x); break;
+  // inform zoom window on panel
+  MuniFullTuneEvent t(EVT_FULLTUNE);
+  t.tone = tone;
+  t.itt = itt;
+  t.pal = pal;
+  t.colour = color;
+  wxQueueEvent(handler,t.Clone());
+}
 
-  case ID_PALETTE_TYPE:     pal.SetPalette(e.GetString()); break;
-  case ID_PALETTE_NEGATIVE: pal.SetNegative(e.GetInt()); break;
+void MuniDisplayCanvas::UpdateScale(bool down, int s)
+{
+  shrinking = true;
+  int oldshrink = shrink;
 
-  case ID_ZOOM_SCALE: zoom = e.x > 0.0 ? e.x : zoomfit; fit = e.fit; break;
-
-  case ID_RESET: Reset(); break;
-
-  default: wxLogDebug("WARNING: MuniDisplay::OnTuneFine - unknown ID");
+  if( down ) {
+    shrink = s;
+    zoom = 1;
+    wxLogDebug("MuniDisplayCanvas::OnTuneFine shrink: 1/%d",shrink);
+  }
+  else {
+    shrink = 1;
+    zoom = s;
+    wxLogDebug("MuniDisplayCanvas::OnTuneFine zoom: %d x",zoom);
   }
 
-  if( e.GetId() == ID_ZOOM_SCALE ) {
+  rendering = oldshrink != 1;
+  wxLogDebug("MuniDisplayCanvas::UpdateScale: %d %d %d %d %d",
+	     oldshrink,zoom,completed,rendering,zooming);
 
-    int x0,y0;
-    GetViewStart(&x0,&y0);
+  UpdateCanvas(shrink,zoom);
+  UpdateScroll();
+}
 
-    int wc,hc;
-    GetClientSize(&wc,&hc);
+void MuniDisplayCanvas::UpdateScroll()
+{
+  if( ! canvas.IsOk() ) return;
 
-    int ux,uy;
-    GetScrollPixelsPerUnit(&ux,&uy);
-
-    // save old position of center
-    double xcold = (x0 + wc/2 - xoff)/(hdu.Width()*oldzoom);
-    double ycold = (y0 + hc/2 - yoff)/(hdu.Height()*oldzoom);
-
-    int w = wxMax(int(zoom*hdu.Width()),1);
-    int h = wxMax(int(zoom*hdu.Height()),1);
-
-    int x = wxMax(int(hdu.Width()*zoom*xcold+xoff-wc/2),0);
-    int y = wxMax(int(hdu.Height()*zoom*ycold+yoff-hc/2),0);
-
-    if( x + wc > w ) x = wxMax(w - wc - 1,0);
-    if( y + hc > h ) y = wxMax(h - hc - 1,0);
-
-    // wxLogDebug("+ %f %f %d %d %d %d %d %d %d %d %d %d %f",xcold,ycold,wc,hc,ux,uy,x/ux,y/uy,x,y,w,h,zoom);
-
-    Scroll(x/ux,y/uy);
-
-    // InitDC must follows (!) Scroll. Why?
-    InitDC(hdu.Width(),hdu.Height(),zoom);
-
-    if( zoom > 1.0 ) {
-      Refresh();
-      return;
-    }
-  }
-
-
-  int x0,y0,wc,hc;
-  GetViewStart(&x0,&y0);
+  int wc,hc;
   GetClientSize(&wc,&hc);
-  if( zoom <= zoomfit  ) {
-    x0 = 0;
-    y0 = 0;
-    wc = zoom*hdu.Width();
-    hc = zoom*hdu.Height();
-  }
+  int w = zoom*canvas.GetWidth();
+  int h = zoom*canvas.GetHeight();
+
+  int dw = -1;
+  int dh = -1;
+  if( w > wc )
+    dw = zoom;
+  if( h > hc )
+    dh = zoom;
+  if( dw > 0 || dh > 0 )
+    SetScrollRate(dw,dh);
+
+  xoff = max((wc - w)/2,0);
+  yoff = max((hc - h)/2,0);
 
   Refresh();
-  rendering = true;
+}
+
+void MuniDisplayCanvas::OnTuneFine(MuniTuneEvent& e)
+{
+  //  wxLogDebug("MuniDisplayCanvas::OnTuneFine %d",e.GetId()==ID_COLOR_NITEVISION);
+
+  bool found = false;
+
+  switch(e.GetId()) {
+
+    // fine tune
+  case ID_ITT_TYPE:   itt.SetItt(e.GetString()); found = true; break;
+  case ID_ITT_AMP:    itt.SetAmp(e.x); found = true; break;
+  case ID_ITT_ZERO:   itt.SetZero(e.x); found = true; break;
+
+  case ID_COLOR_NITEVISION: color.SetNiteVision(e.GetInt()); found = true;break;
+  case ID_COLOR_SATUR:   color.SetSaturation(e.x); found = true; break;
+  case ID_COLOR_HUE:     color.SetHue(e.x); found = true; break;
+  case ID_COLOR_NITETHRESH: color.SetNiteThresh(e.x); found = true; break;
+  case ID_COLOR_NITEWIDTH:  color.SetNiteWidth(e.x); found = true; break;
+  case ID_COLOR_MEAN:    color.SetLevel(e.index,e.x); found = true; break;
+  case ID_COLOR_WEIGHT:  color.SetWeight(e.index,e.x); found = true; break;
+
+  case ID_PALETTE_TYPE:     pal.SetPalette(e.GetString()); found = true; break;
+  case ID_PALETTE_INVERSE: pal.SetInverse(e.GetInt()); found = true; break;
+  }
+  if( found ) {
+    UpdateTune();
+    return;
+  }
+
+  found = false;
+  switch(e.GetId()) {
+
+    // scaling
+  case ID_TONE_BLACK: tone.SetBlack(e.x); found = true; break;
+  case ID_TONE_SENSE:  tone.SetSense(e.x); found = true; break;
+  case ID_TONE_QBLACK: tone.SetQblack(e.x); found = true; break;
+  case ID_TONE_RSENSE:  tone.SetRsense(e.x); found = true; break;
+  }
+  if( found ) {
+    UpdateScale();
+    return;
+  }
+
+  // shrink or zoom
+  if( e.GetId() == ID_ZOOM_SCALE ) {
+    UpdateScale(e.shrink,e.n);
+    return;
+  }
+
+  if( e.GetId() == ID_RESET ) {
+
+    Reset();
+    return;
+  }
+
+  if( e.GetId() == ID_ZOOM ) // ignored
+    return;
+
+  wxLogDebug("WARNING: MuniDisplay::OnTuneFine - reached unknown ID");
 }
 
 void MuniDisplayCanvas::OnAstrometry(MuniAstrometryEvent& e)
@@ -1117,44 +1109,59 @@ void MuniDisplayCanvas::RemoveLayers(int id)
   Refresh();
 }
 
-void MuniDisplayCanvas::OnConfigUpdated(wxCommandEvent& event)
+int MuniDisplayCanvas::GetFitShrink() const
 {
-  display.SetTemperature(config->color_temperature);
-  display.SetColorspace(config->display_colorspace);
-  color.SetWhitePoint(config->color_uwhitepoint,config->color_vwhitepoint);
-  rendering = true;
+  if( ! array.IsOk() ) return 1;
+
+  const int nratios = 6;
+  const int ratios[6] = { 1, 2, 3, 4, 6, 8};
+
+  wxSize client = topwin->GetClientSize();
+
+  int n = ratios[nratios-1];
+  int width = array.Width();
+  int height = array.Height();
+  for(int l = 0; l < nratios; l++) {
+    width = array.Width() / ratios[l];
+    height = array.Height() / ratios[l];
+    if( (width < client.GetWidth() && height < client.GetHeight()) ) {
+      n = ratios[l];
+      break;
+    }
+  }
+  return n;
 }
 
-double MuniDisplayCanvas::GetFitZoom() const
+FitsZoom MuniDisplayCanvas::GetZoom() const
 {
-  if( ! hdu.IsOk() ) return 1.0;
-
-  wxSize isize(hdu.Width(),hdu.Height());
-  wxSize csize = GetClientSize();
-
-  if( isize.GetWidth() > 0 && isize.GetHeight() > 0 ) {
-    double sx = double(csize.GetWidth())/double(isize.GetWidth());
-    double sy = double(csize.GetHeight())/double(isize.GetHeight());
-    return  sx > sy ? sy : sx;
-  }
-  else if( isize.GetWidth() > 0 && isize.GetHeight() == 0 ) {
-    double sx = double(csize.GetWidth())/double(isize.GetWidth());
-    return  sx;
-  }
-
-  return 1.0;
+  if( shrink > 0 )
+    return FitsZoom(shrink);
+  else if( zoom > 0 )
+    return FitsZoom(zoom,ZOOM_ZOOM);
+  else
+    return FitsZoom(1);
 }
 
-
-void MuniDisplayCanvas::SetInitZoom(double z)
+void MuniDisplayCanvas::OnMenuPalauto(wxCommandEvent& event)
 {
-  if( z > 0.0 )
-    zoom = z;
-  else {
-    double zz = GetFitZoom();
-    zoom = zz < 1.0 ? zz : 1.0;
-  }
-  zooming = true;
+  palauto = event.IsChecked();
+  if( palauto )
+    SelectAutoPal();
+  else
+    pal.SetPalette(config->display_pal);
+  UpdateTune();
+}
+
+void MuniDisplayCanvas::SelectAutoPal()
+{
+  //  wxLogDebug("MuniDisplayCanvas::SelectAutoPal");
+  wxString filter(array.GetKey(config->fits_key_filter));
+  if( filter.Find("R") != wxNOT_FOUND )
+    pal.SetPalette(PAL_HEAT);
+  else if( filter.Find("V") != wxNOT_FOUND|| filter.Find("G") != wxNOT_FOUND)
+    pal.SetPalette(PAL_SPRING);
+  else if( filter.Find("B") != wxNOT_FOUND )
+    pal.SetPalette(PAL_COOL);
 }
 
 /*
@@ -1238,153 +1245,3 @@ void MuniDisplayCanvas::OverlayBitmap(const vector<MuniDrawBase *>& drawobj)
   // overlay.push_back(wxBitmap(i));
 
  //}
-
-
-
-// ---- ImageRender ---------------------------------
-
-ImageRender::ImageRender(wxEvtHandler *eh, const FitsImage& fi,
-			 const FitsImage& fis, const FitsDisplay& d,
-			 double zz, bool z, int x, int y, int w, int h):
-  wxThread(wxTHREAD_DETACHED),handler(eh),fitsimage(fi), fimage(fis),
-  display(d), zoom(zz), zooming(z),x0(x),y0(y),width(w),height(h)
-{
-  wxASSERT(handler && fitsimage.IsOk() && x >= 0 && y >= 0 && h > 0 && w > 0);
-  //  wxLogDebug("ImageRender::ImageRender %f %d %d %d",zoom,(int)zooming,
-  //	     (int)fimage.IsOk() ? fimage.GetWidth():0,
-  //	     (int)fimage.IsOk() ? fimage.GetHeight():0);
-}
-
-ImageRender::~ImageRender()
-{
-
-  wxCriticalSectionLocker enter(static_cast<MuniDisplayCanvas *>(handler)->renderCS);
-  static_cast<MuniDisplayCanvas *>(handler)->render = 0;
-
-  //  wxLogDebug("ImageRender::~ImageRender");
-}
-
-wxThread::ExitCode ImageRender::Entry()
-{
-  bool test_destroy = false;
-
-#ifdef __WXDEBUG__
-  wxStopWatch sw;
-#endif
-
-  // length of square side
-  // This value is choosed to be the duration of a single iteration
-  // shorter than an user reaction time (~0.1 sec). Note that
-  // short sides implicates frequent generation of events and memory
-  // allocations. So the value must by determined carefully.
-  const int side = 3*137;
-
-  FitsImage newimage;
-
-  if( false /*fimage.IsOk() && ! zooming*/ )
-    newimage = fimage;
-
-  else {
-    zooming = true;
-    fimage = fitsimage;
-    newimage = fitsimage;//.GetSubImage(x0,y0,width,height);
-    //    newimage.Rescale(zoom);
-  }
-
-
-  //  wxLogDebug("Init took: %d msec",(int)sw.Time());
-
-  int w = fimage.GetWidth();
-  int h = fimage.GetHeight();
-  w = width;
-  h = height;
-  //  newimage = FitsImage(w,h);
-
-  //  int nw = width;
-  int htotal = height*zoom;
-
-
-  int dx = 0, dy = 0;
-
-  int x = 0;
-  int y = 0;
-
-  int imax = wxMin(x0 + w,fitsimage.GetWidth());
-  int jmax = wxMin(y0 + h,fitsimage.GetHeight());
-
-  //  for(int j = 0; j < h ; j += side) {
-  for(int j = y0; j < jmax; j += side) {
-
-    x = 0;
-
-    //    for(int i = 0; i < w; i += side) {
-    for(int i = x0; i < imax; i += side) {
-
-      //      wxLogDebug("Piece rendering took: %d msec",(int)sw.Time());
-
-      if( TestDestroy() ) {
-	test_destroy = true;
-	//	wxLogDebug("Rendering took when delete: %f sec",sw.Time()/1000.0);
-	goto breake;
-      }
-
-      //      wxASSERT(w > i && h > j);
-
-      //      wxASSERT(x0 <= i && i <
-      dx = wxMin(imax - i,side);
-      dy = wxMin(jmax - j,side);
-
-      //      wxLogDebug("%d %d %d %d",dx,dy,i,j);
-      wxASSERT(dx > 0 && dy > 0 && fimage.IsOk());
-
-      FitsImage sub(fimage.GetSubImage(i,j,dx,dy));
-
-      if( zooming ) {
-	FitsImage z(sub.Scale(zoom));
-	sub = z;
-	//	wxASSERT(dx == sub.GetWidth() && dy == sub.GetHeight());
-	dx = wxMin(dx,sub.GetWidth());
-	dy = wxMin(dy,sub.GetHeight());
-      }
-
-      //      wxLogDebug("sub: %d %d %d %d",x,y,newimage.GetWidth(),newimage.GetHeight());
-      wxASSERT(sub.IsOk());
-      //      newimage.SetSubImage(x,y,sub);
-
-      wxASSERT(sub.IsOk());
-
-      MuniRenderEvent ev(EVT_RENDER,ID_SUBRENDER);
-      ev.picture = display.GetImage(sub);
-      ev.x = x;
-      ev.y = htotal - y - dy;
-      ev.x0 = x0;
-      ev.y0 = y0;
-      ev.width = dx;
-      ev.height = dy;
-      //            ev.y = newimage.GetHeight() - y - dy;
-      wxQueueEvent(handler,ev.Clone());
-
-      //      wxASSERT(dx > 0);
-      x = x + dx;
-    }
-    //    wxASSERT(dy > 0);
-    y = y + dy;
-  }
-
- breake:
-
-  MuniRenderEvent ev(EVT_RENDER,ID_RENDER);
-  if( test_destroy )
-    ev.image = FitsImage();
-  else {
-    ev.image = FitsImage(newimage);
-    ev.x0 = x0;
-    ev.y0 = y0;
-    ev.width = width;
-    ev.height = height;
-  }
-  wxQueueEvent(handler,ev.Clone());
-
-  //  wxLogDebug("Rendering took: %f sec",sw.Time()/1000.0);
-  return (wxThread::ExitCode) 0;
-}
