@@ -2,7 +2,7 @@
 
   xmunipack - display image
 
-  Copyright © 2009-2013, 2017 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2009-2013, 2017, 2019 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -68,62 +68,71 @@ MuniDisplay::MuniDisplay(wxWindow *w, MuniConfig *c):
        ID_MENU_FULLSCREEN);
   Bind(EVT_TUNE,&MuniDisplayCanvas::OnTuneFine,
        static_cast<MuniDisplayCanvas *>(canvas));
+  Bind(EVT_FULLTUNE,&MuniDisplayPanel::OnFullTune,
+       static_cast<MuniDisplayPanel *>(panel));
   Bind(EVT_ASTROMETRY,&MuniDisplayCanvas::OnAstrometry,
        static_cast<MuniDisplayCanvas *>(canvas));
   Bind(EVT_PHOTOMETRY,&MuniDisplayCanvas::OnPhotometry,
        static_cast<MuniDisplayCanvas *>(canvas));
-  Bind(EVT_CONFIG_UPDATED,&MuniDisplayCanvas::OnConfigUpdated,
-       static_cast<MuniDisplayCanvas *>(canvas));
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniDisplayCanvas::OnMenuPalauto,
+       static_cast<MuniDisplayCanvas *>(canvas),ID_MENU_PALAUTO);
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniDisplayCanvas::OnValueType,
+       static_cast<MuniDisplayCanvas *>(canvas),ID_VALTYPE);
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniDisplayCanvas::OnCooType,
+       static_cast<MuniDisplayCanvas *>(canvas),ID_COOTYPE);
+
+  //  Bind(EVT_CONFIG_UPDATED,&MuniDisplayCanvas::OnConfigUpdated,
+  //       static_cast<MuniDisplayCanvas *>(canvas));
 }
 
 void MuniDisplay::OnClose(wxCloseEvent& event)
 {
-  wxLogDebug("MuniDisplay::OnClose");
+  //  wxLogDebug("MuniDisplay::OnClose");
   event.Skip();
 }
 
-bool MuniDisplay::SetHdu(const FitsHdu& hdu, const wxImage& icon)
+bool MuniDisplay::SetHdu(const FitsHdu& hdu, const wxImage& icon, bool palauto)
 {
   array = FitsArray(hdu);
   thumb = icon;
 
-  // histogram + itt
-  FitsArrayStat stat(array.IsColor() ? array.Plane(1) : array);
-  double xmin = stat.Med() - 5.0*stat.Mad();
-  double xmax = stat.Med() + 7.0*stat.Mad();
-  hist = FitsHisto(array,xmin,xmax);
-
   wxASSERT(array.IsOk() && icon.IsOk());
   if( ! (array.IsOk() && icon.IsOk()) ) return false;
 
   wxASSERT(static_cast<MuniDisplayCanvas *>(canvas));
   wxASSERT(static_cast<MuniDisplayPanel *>(panel));
 
-  static_cast<MuniDisplayPanel *>(panel)->SetHdu(array,hist);
-  static_cast<MuniDisplayCanvas *>(canvas)->SetHdu(array,icon,stat);
+  static_cast<MuniDisplayPanel *>(panel)->SetArray(array);
+  static_cast<MuniDisplayCanvas *>(canvas)->SetHdu(array,icon,palauto);
 
   return true;
 }
 
 
-bool MuniDisplay::IsRendering() const
-{
-  wxASSERT(static_cast<MuniDisplayCanvas *>(canvas));
-  return static_cast<MuniDisplayCanvas *>(canvas)->IsRendering();
-}
-
 void MuniDisplay::StopRendering()
 {
   wxASSERT(static_cast<MuniDisplayCanvas *>(canvas));
   static_cast<MuniDisplayCanvas *>(canvas)->StopRendering();
 }
 
-double MuniDisplay::GetZoom() const
+int MuniDisplay::GetFitShrink() const
+{
+  wxASSERT(static_cast<MuniDisplayCanvas *>(canvas));
+  return static_cast<MuniDisplayCanvas *>(canvas)->GetFitShrink();
+}
+
+FitsZoom MuniDisplay::GetZoom() const
 {
   wxASSERT(static_cast<MuniDisplayCanvas *>(canvas));
   return static_cast<MuniDisplayCanvas *>(canvas)->GetZoom();
 }
 
+FitsTone MuniDisplay::GetTone() const
+{
+  wxASSERT(static_cast<MuniDisplayCanvas *>(canvas));
+  return static_cast<MuniDisplayCanvas *>(canvas)->GetTone();
+}
+
 FitsItt MuniDisplay::GetItt() const
 {
   wxASSERT(static_cast<MuniDisplayCanvas *>(canvas));
@@ -148,12 +157,6 @@ wxImage MuniDisplay::GetImage()
   return static_cast<MuniDisplayCanvas *>(canvas)->GetImage();
 }
 
-bool MuniDisplay::GetFit() const
-{
-  wxASSERT(static_cast<MuniDisplayCanvas *>(canvas));
-  return static_cast<MuniDisplayCanvas *>(canvas)->GetFit();
-}
-
 void MuniDisplay::ShowPanel(bool show)
 {
   wxASSERT(panel);
@@ -178,12 +181,6 @@ void MuniDisplay::OnLeaveWin(wxMouseEvent& event)
   wxQueueEvent(GetParent(),event.Clone());
 }
 
-double MuniDisplay::GetFitZoom() const
-{
-  wxASSERT(canvas);
-  return static_cast<MuniDisplayCanvas *>(canvas)->GetFitZoom();
-}
-
 /*
 void MuniDisplay::SetOverlay(wxInputStream& istream)
 {
@@ -408,15 +405,23 @@ void MuniDisplay::ShowSources(bool show, const FitsFile& fits)
   wxASSERT(canvas);
 
   if( show ) {
+
+    bool apext = false;
+    for(size_t i = 0; i < fits.HduCount(); i++) {
+      if( fits.Hdu(i).GetExtname() == APEREXTNAME )
+	apext = true;
+    }
+
     for(size_t i = 0; i < fits.HduCount(); i++) {
       if( fits.Hdu(i).GetExtname() == APEREXTNAME ||
-	  fits.Hdu(i).GetExtname() == PHCALNAME ) {
+	  fits.Hdu(i).GetExtname() == PHCALNAME ||
+         (fits.Hdu(i).GetExtname() == FINDEXTNAME && !apext) ) {
 	FitsTable table(fits.Hdu(i));
 	const double *x = table.GetColumn(FITS_LABEL_X).GetCol_double();
 	const double *y = table.GetColumn(FITS_LABEL_Y).GetCol_double();
 	double *f = 0;
 	if( fits.Hdu(i).GetExtname() == APEREXTNAME )
-	  f = (double *) table.GetColumn(FITS_LABEL_APCOUNT "1").GetCol_double();
+	  f = (double *)table.GetColumn(FITS_LABEL_APCOUNT "1").GetCol_double();
 	else if( fits.Hdu(i).GetExtname() == PHCALNAME )
 	  f = (double *) table.GetColumn(FITS_COL_PHOTONS).GetCol_double();
 
@@ -426,9 +431,13 @@ void MuniDisplay::ShowSources(bool show, const FitsFile& fits)
 	  ycoo.push_back(y[i]);
 	  if( f )
 	    flux.push_back(f[i]);
+	  else
+	    flux.push_back(-1.0);
 	}
 
 	MuniPhotometryLayer layer;
+	if( fits.Hdu(i).GetExtname() == APEREXTNAME )
+	  layer.SetHWHM(fits.Hdu(i).GetKeyDouble("HWHM"));
 	layer.DrawObjects(xcoo,ycoo,flux);
 	static_cast<MuniDisplayCanvas *>(canvas)->AddLayer(layer.GetLayer());
       }
@@ -444,13 +453,17 @@ void MuniDisplay::ShowTune(bool show)
   if( ! tune ) {
 
     if( array.IsColor() )
-      tune = new MuniTune(this,ID_TUNE,GetPosition(),wxDefaultSize,config,array,GetItt(),GetColor());
+      tune = new MuniTune(this,ID_TUNE,wxDefaultPosition,wxDefaultSize,
+			  config->icon_size,array,GetTone(),GetItt(),
+			  GetColor());
     else
-      tune = new MuniTune(this,ID_TUNE,GetPosition(),wxDefaultSize,config,array,GetItt(),GetPalette());
-
-    tune->Show(true);
+      tune = new MuniTune(this,ID_TUNE,wxDefaultPosition,wxDefaultSize,
+			  config->icon_size,array,GetTone(),GetItt(),
+			  GetPalette());
 
     Bind(wxEVT_CLOSE_WINDOW,&MuniDisplay::OnCloseTune,this,tune->GetId());
+
+    tune->Show(true);
   }
 }
 
