@@ -2,7 +2,7 @@
 
   XMunipack - derived events
 
-  Copyright © 2009-2012, 2017 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2009-2012, 2017-9 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -48,12 +48,11 @@ wxDECLARE_EVENT(EVT_FINISH_DIALOG, wxCommandEvent);
 
 class MuniSlewEvent: public wxEvent
 {
-public:
-  MuniSlewEvent(wxEventType eventType =wxEVT_NULL, int winid =wxID_ANY):
-    wxEvent(winid, eventType),z_x(0),z_y(0),i_x(0),i_y(0) {}
+ public:
+ MuniSlewEvent(wxEventType eventType =wxEVT_NULL, int id =wxID_ANY):
+  wxEvent(id, eventType),x(0),y(0) {}
   virtual wxEvent *Clone(void) const { return new MuniSlewEvent(*this); }
-  int z_x, z_y, i_x, i_y;
-  FitsBitmap picture;
+  int x,y;
 };
 
 wxDECLARE_EVENT(xEVT_SLEW,MuniSlewEvent);
@@ -65,18 +64,39 @@ wxDECLARE_EVENT(xEVT_SLEW,MuniSlewEvent);
 class MuniTuneEvent: public wxCommandEvent
 {
  public:
-  MuniTuneEvent(WXTYPE e =wxEVT_NULL, int id =wxID_ANY): wxCommandEvent(e,id),x(0.0),index(0),fit(true) {}
-  MuniTuneEvent(wxCommandEvent& e): wxCommandEvent(e),x(0.0),index(0),fit(true) {}
+ MuniTuneEvent(WXTYPE e =wxEVT_NULL, int id =wxID_ANY):
+  wxCommandEvent(e,id),x(0.0),n(0),index(0),shrink(true) {}
+ MuniTuneEvent(wxCommandEvent& e):
+  wxCommandEvent(e),x(0.0),n(0),index(0),shrink(true) {}
   wxEvent *Clone(void) const { return new MuniTuneEvent(*this); }
   double x;
+  int n;
   int index;
-  bool fit;
+  bool shrink;
 };
 
 wxDECLARE_EVENT(EVT_TUNE,MuniTuneEvent);
 
 #define MuniTuneEventHandler(func) (&func)
 
+// -- Full Tune
+
+class MuniFullTuneEvent: public wxCommandEvent
+{
+ public:
+ MuniFullTuneEvent(WXTYPE e=wxEVT_NULL,int id=wxID_ANY): wxCommandEvent(e,id) {}
+ MuniFullTuneEvent(wxCommandEvent& e): wxCommandEvent(e) {}
+  wxEvent *Clone(void) const { return new MuniFullTuneEvent(*this); }
+  FitsTone tone;
+  FitsItt itt;
+  FitsPalette pal;
+  FitsColor colour;
+};
+
+wxDECLARE_EVENT(EVT_FULLTUNE,MuniFullTuneEvent);
+
+#define MuniFullTuneEventHandler(func) (&func)
+
 // -- Astrometry
 
 class MuniAstrometryEvent: public wxCommandEvent
@@ -135,13 +155,16 @@ wxDECLARE_EVENT(EVT_HDU,MuniHduEvent);
 class MuniRenderEvent: public wxThreadEvent
 {
  public:
-  MuniRenderEvent(WXTYPE commandEventType =wxEVT_NULL, int id =wxID_ANY):
-    wxThreadEvent(commandEventType,id),x(0),y(0) {}
-  MuniRenderEvent(wxThreadEvent& e): wxThreadEvent(e),x(0),y(0) {}
+ MuniRenderEvent(WXTYPE commandEventType =wxEVT_NULL, int i =wxID_ANY):
+  wxThreadEvent(commandEventType,i),id(0),completed(false),x(0),y(0) {}
+ MuniRenderEvent(wxThreadEvent& e):
+  wxThreadEvent(e),id(0),completed(false),x(0),y(0) {}
   wxEvent *Clone(void) const { return new MuniRenderEvent(*this); }
+  int id;
+  bool completed;
   FitsBitmap picture;
   FitsImage image;
-  int x,y,x0,y0,width,height;
+  int x,y;
 };
 
 wxDECLARE_EVENT(EVT_RENDER,MuniRenderEvent);
@@ -160,6 +183,8 @@ class FitsOpenEvent: public wxThreadEvent
   wxString filename;
   FitsFile fits;
   FitsMeta meta;
+  std::vector<FitsTone> tones;
+  std::vector<wxImage> icons;
 };
 
 
