@@ -2,7 +2,7 @@
 
   xmunipack - fits implementation
 
-  Copyright © 1997 - 2014, 2017 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 1997 - 2014, 2017-9 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -281,6 +281,16 @@ long FitsHdu::Height() const
   return Naxes(1);
 }
 
+long FitsHdu::GetWidth() const
+{
+  return Naxes(0);
+}
+
+long FitsHdu::GetHeight() const
+{
+  return Naxes(1);
+}
+
 bool FitsHdu::IsOk() const
 {
   return ! header.IsEmpty();
@@ -332,14 +342,24 @@ FitsArrayData::FitsArrayData(int n, long *ns, float *a):
   naxis(n),naxes(ns),array(a) {}
 
 
-FitsArrayData::FitsArrayData(const FitsArrayData& copy)
-{
-  wxFAIL_MSG("FitsArrayData ----- WE ARE REALY NEED COPY CONSTRUCTOR ---");
+FitsArrayData::FitsArrayData(const FitsArrayData& other)
+{  // COPY constructor
+  long n = 1;
+  for(long i = 0; i < other.naxis; i++)
+    n = n * other.naxes[i];
+  naxis = other.naxis;
+  naxes = new long[naxis];
+  array = new float[n];
+  copy(other.naxes,other.naxes+naxis,naxes);
+  copy(other.array,other.array+n,array);
 }
 
 FitsArrayData& FitsArrayData::operator = (const FitsArrayData& other)
 {
  wxFAIL_MSG("*** FitsArrayData: WE ARE REALLY NEED ASSIGNMENT CONSTRUCTOR");
+ if( this != & other ) {
+   ;
+ }
  return *this;
 }
 
@@ -369,19 +389,24 @@ FitsArray::FitsArray(const FitsHdu& h, int n, long *ns, float *a): FitsHdu(h)
 }
 
 
+
 FitsArray::FitsArray(const FitsHdu& h): FitsHdu(h)
 {
   wxASSERT(h.Type() == HDU_IMAGE && h.IsOk());
 
   FitsArrayData *data = static_cast<FitsArrayData *>(GetRefData());
   wxASSERT(data);
-
   npixels = 1;
   for(int k = 0; k < data->naxis; k++)
     npixels = npixels*data->naxes[k];
 }
 
-FitsArray::~FitsArray() {}
+/*
+FitsArray::~FitsArray()
+{
+  UnRef();
+}
+*/
 
 wxObjectRefData *FitsArray::CreateRefData() const
 {
@@ -393,27 +418,12 @@ wxObjectRefData *FitsArray::CloneRefData(const wxObjectRefData *that) const
 {
   const FitsArrayData *olddata = static_cast<const FitsArrayData *>(that);
   wxASSERT(olddata);
-
-  FitsArrayData *newdata = new FitsArrayData;
-  newdata->naxis = olddata->naxis;
-  newdata->naxes = new long[olddata->naxis];
-  memcpy(newdata->naxes,olddata->naxes,olddata->naxis);
-  newdata->array = new float[npixels];
-  memcpy(newdata->array,olddata->array,npixels*sizeof(float));
-  return newdata;
-}
-
-FitsArray FitsArray::Copy() const
-{
-  FitsArray new_array;
-  new_array.m_refData = CloneRefData(m_refData);
-  return new_array;
+  return new FitsArrayData(*olddata);
 }
 
 bool FitsArray::IsOk() const
 {
   FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
-  //  wxASSERT(data);
   if( ! data ) return false;
   return m_refData && data->naxis > 0 && data->naxes && data->array;
 }
@@ -437,6 +447,18 @@ long FitsArray::Naxes(int n) const
     return 0;
 }
 
+long* FitsArray::Naxes() const
+{
+  FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
+  wxASSERT(data);
+
+  long *ns = new long[data->naxis];
+  for(int n = 0; n < data->naxis; n++)
+    ns[n] = data->naxes[n];
+  return ns;
+}
+
+
 const float *FitsArray::PixelData() const
 {
   FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
@@ -447,235 +469,27 @@ const float *FitsArray::PixelData() const
 }
 
 
-float FitsGeometry::MeanLine(float x, float w)
-{
-  FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
-  wxASSERT(data && data->array && data->naxes && data->naxis == 1);
-
-  int i0 = int(x + 0.5);
-  int dx = int(w/2.0 - 0.5);
-
-  int i1 = max(i0 - dx,0);
-  int i2 = min(long(i0 + dx),data->naxes[0]);
-
-  float s = 0;
-  float n = 0;
-  for(int i = i1; i <= i2; i++) {
-    // s = s + Pixel(i);
-    // wxASSERT(0 <= i && i < data->npixels);
-    s = s + *(data->array + i);
-    n = n + 1;
-  }
-
-  if( n > 0 ) return s/n; else return 0.0;
-}
-
-
-float FitsGeometry::MeanRect(float x, float y, float w, float h)
-{
-  FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
-  wxASSERT(data && data->array && data->naxes && data->naxis == 2);
-
-  /*
-  const int dmax = max(w,h)/2;
-
-  int i0 = int(x + 0.5);
-  int j0 = int(y + 0.5);
-  int dx = min(int(w/2.0 - 0.5),dmax);
-  int dy = min(int(h/2.0 - 0.5),dmax);
-
-  wxASSERT(dx >= 0 && dy >= 0);
-
-  int i1 = max(i0 - dx,0);
-  int j1 = max(j0 - dy,0);
-  int i2 = min(long(i0 + dx),data->naxes[0]-1);
-  int j2 = min(long(j0 + dy),data->naxes[1]-1);
-  */
-
-  int i0 = int(x + 0.5);
-  int j0 = int(y + 0.5);
-  int dx = int(w/2.0 - 0.5);
-  int dy = int(h/2.0 - 0.5);
-
-  wxASSERT(dx >= 0 && dy >= 0);
-
-  int i1 = max(i0 - dx,0);
-  int j1 = max(j0 - dy,0);
-  int i2 = min(long(i0 + dx),data->naxes[0]-1);
-  int j2 = min(long(j0 + dy),data->naxes[1]-1);
-
-
-  // !!!!!!!!!!!!!!!!!!!!!!!!!!!!
-
-  //  if( dx > 0 && dy > 0 )
-  //    wxLogDebug(_("%d %d"),dx,dy);
-
-//   int ww = i2 - i1 + 1;
-//   int hh = j2 - j1 + 1;
-//   ww = dx;
-//   hh = dy;
-//   if(  ww > 0 && hh > 0 ) {
-//     float a[ww*hh];
-//     const float *array = data->array;
-//     wxASSERT(array);
-//     for(int j = 0; j < hh; j++)
-//       memcpy(a+j*ww,array+(j1+j)*data->naxes[0]+i1,ww*sizeof(float));
-
-//     //  return a[0];
-//     return FitsArrayStat::QMed(ww*hh,a,max((ww*hh)/2,0));
-//   }
-//   else
-//     return Pixel(i1,j1);
-
-
-  ///!!!!!!!!!!!!!!!!!!!!!!!!!!!
-
-  float s = 0.0;
-
-#ifdef __WXDEBUG__
-  int n = 0;
-#else
-  int n = (j2 - j1 + 1)*(i2 - i1 + 1);
-#endif
-
-  float *ai = data->array + i1;
-  int idim = data->naxes[0];
-  for(int j = j1; j <= j2; j++) {
-    //    float *a = data->array + j*data->naxes[0] + i1;
-    float *a = ai + j*idim;
-    for(int i = i1; i <= i2; i++) {
-      //      s = s + Pixel(i,j);
-      //      wxASSERT(0 <= i && i < data->npixels);
-      s += *a++;
-      //      s = s + *(a + i);
-      //      wxLogDebug("%f",(float)*(a + i));
-#ifdef __WXDEBUG__
-      n = n + 1;
-#endif
-    }
-  }
-#ifdef __WXDEBUG__
-  int nx = j2 - j1;
-  int ny = i2 - i1;
-  int nn = (nx + 1)*(ny + 1);
-  if( n != nn )
-    {wxLogDebug("%d %d %d %d",n,nn,nx,ny);}
-#endif
-  wxASSERT(n == (j2 - j1 + 1)*(i2 - i1 +1) );
-  if( n > 0 ) return s/n; else return 0.0;
-}
-
 
 FitsArray FitsArray::Plane(int n) const
 {
-  int np = Naxis() - 1;
-  wxASSERT(np > 0);
+  wxASSERT(0 <= n && n < 3);
 
-  long npixels = 1;
+  int np = 2;
   long *ns = new long[np];
   ns[0] = Naxes(0);
   ns[1] = Naxes(1);
-  npixels = ns[0]*ns[1];
-  /*
-  int k = 0;
-  for(int l = 0; l < Naxis(); l++)
-    if( l != n ) {
-      int m = Naxes(l);
-      ns[k++] = m;
-      npixels = npixels*m;
-    }
-  */
+  long npixels = ns[0]*ns[1];
 
   const FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
-  wxASSERT(data);
-  const float *array = data->array;
-  wxASSERT(array);
+  wxASSERT(data && data->array);
+  const float *array = data->array + n*npixels;
 
   float *a = new float[npixels];
-  //  k = 0;
-  //  for(int l = 0; l < Naxis(); l++)
-  //    if( l != n )
-  memcpy(a,array+n*npixels,npixels*sizeof(float));
-
-    //  for(int l = 0; l < npixels; l++) a[l] = 1;
-
+  copy(array,array+npixels,a);
   return FitsArray(*this,np,ns,a);
 }
 
 
-FitsArrayStat::FitsArrayStat(const FitsArray& a, int nestim): FitsArray(a)
-{
-  min = 0.0;
-  max = 1.0;
-  Statistics(nestim);
-}
-
-
-void FitsArrayStat::Statistics(int ndata)
-{
-  FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
-  wxASSERT(data && data->array && data->naxes);
-
-  int bstep = ndata > 0 ? Npixels()/ndata + 1 : 13;
-  int nd = Npixels()/(bstep*bstep);
-
-  float *d = new float[nd+1];
-
-  // median
-  for(int i = 0, j = 0; j < nd; i += bstep, j++) {
-    float x = Pixel(i);
-    d[j] = x;
-    if( x > max ) max = x;
-    if( x < min ) min = x;
-  }
-  med = QMed(nd,d,nd/2+1);
-
-  // mad = median of absolute deviations
-  int n = 0;
-  int imax = Npixels() - bstep;
-  for(int i = 0; n < nd && i < imax; i += bstep) {
-    float r = Pixel(i) - med;
-    if( r > 0.0 )
-      d[n++] = r;
-  }
-  mad = QMed(n,d,n/2+1);
-
-  delete[] d;
-
-  //  wxLogDebug(_("%f %f %d %f %d"),mad,med,n,Pixel(1000),bstep);
-}
-
-
-float FitsArrayStat::QMed(int n, float *a, int k)
-{
-  float w,x;
-  int l,r,i,j;
-
-  x = 0.0;
-
-  l = 0;
-  r = n - 1;
-
-  while( l < r ) {
-    x = a[k]; i = l; j = r;
-
-    do {
-
-      while( a[i] < x && i < r) i++;
-      while( x < a[j] && j > l) j--;
-
-      if( i <= j ) { w = a[i]; a[i] = a[j]; a[j] = w; i++; j--; }
-
-    } while ( i <= j );
-
-    if( j < k ) l = i;
-    if( k < i ) r = j;
-  }
-
-  return(x);
-}
-
-
 int FitsArray::Flavour() const
 {
   FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
@@ -1473,10 +1287,7 @@ FitsFile::FitsFile(const wxString& name):
 	fits_read_img(f,TFLOAT,firstelem,ndata,&nullval,image,&dummy,&stat);
 	if( stat ) { delete[] naxes; delete[] image; goto crash; }
 
-	//	FitsArray array(head,naxis,naxes,image);
-	//	hdu.push_back(array);
 	hdu.push_back(FitsArray(head,naxis,naxes,image));
-
       }
       else {
 	hdu.push_back(FitsHdu(head));
