@@ -2,7 +2,7 @@
 
   XMunipack - FITS headers
 
-  Copyright © 2009-2013, 2017 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2009-2013, 2017-9 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -28,15 +28,25 @@
 #include <fitsio.h>
 #include <algorithm>
 #include <vector>
+#include <set>
 #include <wx/wx.h>
 #include <wx/string.h>
 #include <wx/arrstr.h>
 #include <wx/filename.h>
+#include <cmath>
 
 #ifdef __WXDEBUG__
 #include <wx/debug.h>
 #endif
 
+#ifdef __WXOSX__
+#define __ADOBERGB__
+#else
+#define __sRGB__
+#endif
+
+
+#define FINDEXTNAME "FIND"
 #define APEREXTNAME "APERPHOT"
 #define PHCALNAME "PHCAL"
 #define FITS_LABEL_X "X"
@@ -61,17 +71,22 @@ enum hdu_flavour {
   HDU_DUMMY
 };
 
+enum zoom_type {
+  ZOOM_FIRST,
+  ZOOM_SHRINK, ZOOM_ZOOM,
+  ZOOM_LAST
+};
+
 enum itt_type {
   ITT_FIRST,
-  ITT_LINE, ITT_ASINH, ITT_LOG, ITT_SQRT, ITT_GAMMA,
-  ITT_NORM, ITT_LOGIS, ITT_ATAN, ITT_SQR,
+  ITT_LINE, ITT_LOGIS, ITT_SQRT, ITT_SQR,
   ITT_LAST
 };
 
-enum itt_kind {
-  ITT_KIND_FIRST,
-  ITT_KIND_ABS, ITT_KIND_REL,
-  ITT_KIND_LAST
+enum tone_kind {
+  TONE_KIND_FIRST,
+  TONE_KIND_ABS, TONE_KIND_REL,
+  TONE_KIND_LAST
 };
 
 enum color_palette {
@@ -91,10 +106,14 @@ enum color_temperature {
   COLORTEMP_D65
 };
 
+enum tune_operation {
+  OP_TUNE_ZOOM=2, OP_TUNE_SCALE=4, OP_TUNE_ITT=8, OP_TUNE_PAL=16,
+  OP_TUNE_COLOUR=32, OP_TUNE_RGB=64
+};
+
 enum units_type {
   UNIT_FIRST,
-  UNIT_COUNT, UNIT_MAG, UNIT_PHOTON, UNIT_FLUX, UNIT_FNU, UNIT_FLAM, UNIT_ABMAG,
-  UNIT_STMAG,
+  UNIT_COUNT, UNIT_PHOTON, UNIT_MAG, UNIT_INTENSITY, UNIT_STMAG,
   UNIT_LAST
 };
 
@@ -150,8 +169,10 @@ public:
   // templates and helpers for derived classes (Array + Table)
   virtual int Naxis() const;
   virtual long Naxes(int n) const;
-  virtual long Width() const;
-  virtual long Height() const;
+  virtual long Width() const; // obsolete
+  virtual long Height() const; // obsolete
+  virtual long GetWidth() const;
+  virtual long GetHeight() const;
   virtual bool IsOk() const;
   virtual bool IsColor() const;
   virtual bool IsModified() const { return modified; }
@@ -204,13 +225,14 @@ public:
   FitsArray();
   FitsArray(const FitsHdu&, int, long *, float *);
   FitsArray(const FitsHdu&);
-  virtual ~FitsArray();
-  FitsArray Copy() const;
+  //  virtual ~FitsArray();
 
   int Naxis() const;
   long Naxes(const int n) const;
+  long* Naxes() const;
   inline float Pixel(int) const;
   inline float Pixel(int, int) const;
+  inline float Pixel(int, int, int) const;
   const float *PixelData() const;
   FitsArray Plane(int) const;
   long Npixels() const { return npixels; }
@@ -244,29 +266,41 @@ inline float FitsArray::Pixel(int i) const
 inline float FitsArray::Pixel(int x, int y) const
 {
   FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
-  /*
   wxASSERT(data && data->array && data->naxes && data->naxis == 2);
   wxASSERT(0 <= x && x < data->naxes[0] && 0 <= y && y < data->naxes[1]);
-  */
   return *(data->array+y*data->naxes[0] + x);
 }
 
+inline float FitsArray::Pixel(int x, int y, int z) const
+{
+  FitsArrayData *data = static_cast<FitsArrayData *>(m_refData);
+  wxASSERT(data && data->array && data->naxes && data->naxis == 3);
+  wxASSERT(0 <= x && x < data->naxes[0] && 0 <= y && y < data->naxes[1] &&
+	   0 <= z && z < data->naxes[2]);
+  return *(data->array + (z*data->naxes[1] + y)*data->naxes[0] + x);
+}
 
 
 class FitsArrayStat: public FitsArray
 {
 public:
   FitsArrayStat(const FitsArray&, int=0);
-  void Statistics(int);
-  float Med() const { return med; }
-  float Mad() const { return mad; }
-  float GetMin() const { return min; }
-  float GetMax() const { return max; }
+
+  void InitStat();
+
+  float Med() const { return GetMed(); }
+  float Mad() const { return GetMad(); }
+  float GetMed() const;
+  float GetMad() const;
+  float GetMin() const;
+  float GetMax() const;
+  static float QMed(long, float *, int);
 
 private:
 
-  float med, mad, min, max;
-  float QMed(int, float *, int);
+  bool statistics;
+  int skip;
+  float med, mad, xmin, xmax;
 
 };
 
@@ -696,16 +730,29 @@ private:
 
 // image operations
 
+class EmpiricalCDF
+{
+  long ncdf;
+  float *xcdf, *ycdf;
+
+ public:
+  EmpiricalCDF(): ncdf(0),xcdf(0),ycdf(0) {}
+  EmpiricalCDF(long, const float*);
+  EmpiricalCDF(const EmpiricalCDF&);
+  EmpiricalCDF& operator=(const EmpiricalCDF&);
+  virtual ~EmpiricalCDF();
+
+  bool IsOk() const;
+  float GetQuantile(float) const;
+  float GetInverse(float) const;
+};
+
 
 class FitsHisto: public wxObject
 {
 public:
   FitsHisto();
-  FitsHisto(int,int*,float*);
   FitsHisto(const FitsArray& , int=0);
-  FitsHisto(const FitsArray& , float, float, int=0);
-  virtual ~FitsHisto();
-
 
   int NBins() const;
   int Hist(int n) const;
@@ -719,7 +766,6 @@ public:
 private:
 
   float wbin,xmin,xmax;
-  int npixels;
 
   void Create(int, const FitsArrayStat&);
 };
@@ -727,64 +773,73 @@ private:
 
 class FitsGeometry: public FitsArray
 {
-public:
+
+  FitsArray Shrink(int,const long *);
+  FitsArray Zoom(int,const long *);
+
+ public:
   FitsGeometry(const FitsArray&);
 
-  FitsArray Scale(float);
-  FitsArray Scale(const long *);
+  FitsArray Shrink(int);
+  FitsArray Zoom(int);
   FitsArray Scale(int,int);
 
-  FitsArray SubArray(int,int,int,int);
+  FitsArray GetSubArray(int,int,int,int);
+  //virtual FitsImage PutSubImage(int,int,const FitsArray&);
   void SetSubArray(int,int,const FitsArray&);
 
-  float MeanLine (float, float);
-  float MeanRect (float, float, float, float);
+  float MeanLine(int, int);
+  float MeanRect(int, int, int, int);
+  float MeanRect_debug(int, int, int, int);
 };
 
 
+class FitsTone
+{
+  bool initialised;
+  float black, qblack, sense, rsense, refsense;
+  EmpiricalCDF cdf_back;
+
+ public:
+
+  FitsTone();
+  FitsTone(const FitsArray&);
+  FitsTone(long, const float *);
+  bool IsOk() const;
+  void Setup(long, const float *);
+  float Scale(float x) const { return (x - black) / sense; }
+  float *Scale(long, const float *) const;
+  void SetBlack(float);
+  void SetQblack(float);
+  void SetSense(float);
+  void SetRsense(float);
+  float GetSense() const { return sense; }
+  float GetBlack() const { return black; }
+  float GetQblack() const { return qblack; }
+  float GetRsense() const { return rsense; }
+  float GetBlackMin() const;
+  float GetBlackMax() const;
+  void Reset();
+
+};
 
 
 // -- Intensity transfer table (ITT)
-
-
 class FitsItt
 {
 public:
   FitsItt(int =ITT_LINE);
-  FitsItt(int, int, const FitsArrayStat& );
 
   void SetItt(int);
   void SetItt(const wxString&);
-  void SetKind(int);
-  void SetSensitivity(float);
-  void SetBlack(float);
-  void SetBrightness(float);
-  void SetContrast(float);
   void SetAmp(float);
   void SetZero(float);
-  void SetRangeMin(float);
-  void SetRangeMax(float);
-
   int GetItt() const;
   wxString GetItt_str() const;
-  int GetKind() const;
-  float GetSensitivity() const;
-  float GetContrast() const;
-  float GetBlack() const;
   float GetAmp() const;
   float GetZero() const;
-  float GetMed() const;
-  float GetMad() const;
-  float GetRangeMin() const;
-  float GetRangeMax() const;
-
-  void Init(float, float);
-  void Init(const FitsArrayStat&);
   void Reset();
-  unsigned char Fscale(float) const;
-  float Scale(float) const;
-  float *Scale(int, const float *);
-  float InvScale(int) const;
+  float *Scale(long, const float *);
 
   static wxString Type_str(int);
   static wxArrayString Type_str();
@@ -793,20 +848,14 @@ public:
 
 private:
 
-  int itt, kind;
-  float black, sense, contrast, med, mad, amp, zero, rmax, rmin;
+  int itt;
+  float amp, zero;
 
   float (FitsItt::*Func)(float) const;
   float Itt_linear(float) const;
-  float Itt_asinh(float) const;
-  float Itt_gamma(float) const;
-  float Itt_log(float) const;
-  float Itt_norm(float) const;
   float Itt_sqrt(float) const;
   float Itt_square(float) const;
   float Itt_logis(float) const;
-  float Itt_atan(float) const;
-  float invnorm(float) const;
 
 };
 
@@ -816,50 +865,41 @@ class FitsColor: public wxObject
 public:
   FitsColor();
   FitsColor(const wxString&,const FitsArray&);
-  //  virtual ~FitsColor();
 
-  void InitUV();
   void Init(const wxString&,const FitsArray&);
 
   void SetSaturation(float);
   void SetHue(float);
-  //  void SetWhite(float);
-  void SetWhitePoint(float =-1.0,float =-1.0);
-  void SetNightVision(bool);
-  void SetNightThresh(float);
-  void SetNightWidth(float);
+  void SetNiteVision(bool);
+  void SetNiteThresh(float);
+  void SetNiteWidth(float);
   void SetTrans(const wxString&);
   void SetTrans(const wxString&,const wxString&);
   void SetTrans(int,int);
   void SetTrans(int,int,float);
   void SetLevel(int,float);
   void SetWeight(int,float);
+  void Reset();
   float GetWeight(int) const;
   float GetLevel(int) const;
   float GetTrans(int,int) const;
   int GetColors() const;
   int GetBands() const;
-  inline void XYZ_Luv(float,float,float,float,float&, float&, float&);
-  inline void Luv_XYZ(float,float,float,float,float&, float&, float&);
-  void XYZ_Luv(int,float*,float*,float*,float,float*, float*, float*);
-  void Luv_XYZ(int,float*,float*,float*,float,float*, float*, float*);
-  void NightVision(int,float*,float*,float*,float*);
-  void Instr_XYZ(int,size_t,const float **,float*,float*,float*);
-  void TuneColors(int,float*,float*,float*);
+  void XYZ_Lab(long,float*,float*,float*,float*, float*, float*);
+  void Lab_XYZ(long,float*,float*,float*,float*, float*, float*);
+  void Lab_XYZ(float,float,float,float*, float*, float*);
+  void NiteVision(long,float*,float*,float*,float*);
+  void Instr_XYZ(long,size_t,const float **,float*,float*,float*);
+  void TuneColors(long,float*,float*,float*);
 
   float Scotopic(float,float,float);
-  //  void Convert(float, float, float,
-  //	       unsigned char &, unsigned char &, unsigned char &);
-  //  void Transform(size_t, float *, float&, float&, float&);
 
   float GetSaturation() const { return saturation; }
   float GetHue() const { return 57.29577951*hue; }
-  float GetWhitePoint_U() const { return uwhite; }
-  float GetWhitePoint_V() const { return vwhite; }
-  bool GetNightVision() const { return nvision; }
-  float GetNightThresh() const { return nthresh; }
-  float GetNightWidth() const { return nwidth; }
-  float NightProfile(float) const;
+  bool GetNiteVision() const { return nitevision; }
+  float GetNiteThresh() const { return nitethresh; }
+  float GetNiteWidth() const { return nitewidth; }
+  float NiteProfile(float) const;
   wxString GetColorspace() const;
 
   float InvGamma(float);
@@ -869,56 +909,20 @@ public:
 
 private:
 
+  static const float Xn,Yn,Zn;
+
   wxString cspace;
   int ispace;
-  bool nvision;
-  float saturation, hue, nthresh, nwidth/*, Yn*/;
-  float uwhite, vwhite;
+  float saturation, hue;
+  bool nitevision;
+  float nitethresh, nitewidth;
 
+  void Lab_fun(long npix, float *I, float In, float *f);
+  void Lab_invfun(long npix, float *f, float In, float *I);
+  float Lab_invfun(float f);
 };
 
 
-
-
-// http://en.wikipedia.org/wiki/CIELUV_color_space
-inline void FitsColor::XYZ_Luv(float X, float Y, float Z, float Yn,
-			float &L, float &u, float &v)
-{
-  float s = X + 15.0*Y + 3.0*Z;
-  float u1 = 4.0*X/s;
-  float v1 = 9.0*Y/s;
-
-  float yy = Y/Yn;
-  if( yy > 0.0088565 /* = powf(6.0/29.0,3)*/ )
-    L = 116.0*powf(yy,0.3333333) - 16.0;
-  else
-    L = 903.30*yy;   /* = pow(26.0/3.0,3) */
-
-  float t = 13.0*L;
-  u = t*(u1 - uwhite);
-  v = t*(v1 - vwhite);
-}
-
-inline void FitsColor::Luv_XYZ(float L, float u, float v, float Yn,
-			float &X, float &Y, float &Z)
-{
-  float s = 13.0*L;
-  float u1 = u/s + uwhite;
-  float v1 = v/s + vwhite;
-
-  if( L <= 8.0 )
-    Y = L*0.0011071*Yn; /* = powf(3.0/29.0,3) */
-  else {
-    float t = (L + 16.0)/116.0;
-    Y = t*t*t*Yn;
-    //    Y = ipowf((L + 16.0)/116.0,3)*Yn;
-  }
-
-  float t = Y/(4.0*v1);
-  X = t*(9.0*u1);//-9.0*Y*u1/(v1*(u1 - 4.0) - v1*u1);
-  Z = t*(12.0-3.0*u1-20.0*v1);//(9.0*Y - 15.0*Y*v1 - X*v1)/(3.0*v1);
-}
-
 // ---  FitsPalette
 
 class FitsPalette: public wxObject
@@ -929,20 +933,18 @@ public:
 
   void SetPalette(int);
   void SetPalette(const wxString&);
-  void SetNegative(bool=false);
+  void SetInverse(bool=false);
   int GetColors() const;
   int GetPalette() const;
   wxString GetPalette_str() const;
-  bool GetNegative() const;
+  bool GetInverse() const;
+  void Reset();
 
   unsigned char R(int i) const;
   unsigned char G(int i) const;
   unsigned char B(int i) const;
   void RGB(float, unsigned char&, unsigned char&, unsigned char&);
-  unsigned char *RGB(int,float *);
-  const unsigned char *RData() const;
-  const unsigned char *GData() const;
-  const unsigned char *BData() const;
+  unsigned char *RGB(long,float *);
 
   static wxString Type_str(int);
   static wxArrayString Type_str();
@@ -950,7 +952,7 @@ public:
 private:
 
   int pal;
-  bool negative;
+  bool inverse;
 
   void CreatePalette();
 };
@@ -997,9 +999,6 @@ public:
   void RaSix(double,int&,int&,double&);
   void DecSix(double,char&,int&,int&,double&);
 
-  wxString Get_str(int,int);
-  void GetStr(int,int,wxString&,wxString&);
-
   void GetPix(int,int,wxString&,wxString&);
   void GetCoo(int,int,wxString&,wxString&);
 
@@ -1022,14 +1021,14 @@ private:
 class PhotoFilter
 {
 public:
-  PhotoFilter(): flux0(-1.0), leff(0.0), lwidth(0.0) {}
-  PhotoFilter(const wxString& n,double f0, double le, double lw):
-    name(n),flux0(f0),leff(le),lwidth(lw) {}
+ PhotoFilter(): leff(0.0), lwidth(0.0), flam(-1.0) {}
+ PhotoFilter(const wxString& n, double le, double lw, double f0):
+  name(n),leff(le),lwidth(lw),flam(f0) {}
 
-  bool IsOk() const { return flux0 > 0.0; }
+  bool IsOk() const { return flam > 0.0; }
 
   wxString name;
-  double flux0, leff, lwidth;
+  double leff, lwidth, flam;
 
 };
 
@@ -1061,20 +1060,18 @@ public:
 
 class PhotoConv
 {
-  double flux0, leff, lwidth, area, exptime, gain, scale;
-  double cspeed, hplanck, ABspflux, STspflux;
+  double flam, leff, lwidth, area, exptime, scale;
+  static const double cspeed, hplanck, evolt, sqrtpi2, STspflux;
 
+  double mag(double,double) const;
+  double intensity(double) const;
 
  public:
   PhotoConv() {}
-  PhotoConv(const PhotoFilter&, double, double, double, double);
+  PhotoConv(const PhotoFilter&, double, double, double);
 
-  double GetFlux(double) const;
-  double GetFnu(double) const;
-  double GetFlam(double) const;
-  double GetMag(double,double) const;
+  double GetIntensity(double) const;
   double GetMag(double) const;
-  double GetABMag(double) const;
   double GetSTMag(double) const;
 
 };
@@ -1084,17 +1081,15 @@ class FitsValue
 {
 public:
   FitsValue();
-  FitsValue(const FitsArray&, const wxString&, const wxString&, const wxString&,
+  FitsValue(const FitsArray&, const wxString&, const wxString&,
 	    const wxString&, const wxString&);
-  FitsValue(const FitsArray&,const FitsArray&,const FitsArray&);
 
   void SetType(int);
   void SetType(const wxString&);
   units_type GetType() const;
 
-  double Get(double) const;
-  std::vector<double> Get(int,int) const;
   wxString Get_str(int,int) const;
+  wxString Get_str(int,int,int) const;
 
   static wxString Label_str(int);
   static wxArrayString Label_str();
@@ -1105,15 +1100,16 @@ private:
 
   units_type type;
   bool hascal;
-  std::vector<FitsArray> arrays;
-  std::vector<double> gain, area, exptime, scale;
-  std::vector<wxString> filter;
-  wxString photsys, fits_key_area, fits_key_gain, fits_key_exptime, fits_key_filter;
+  FitsArray array;
+  double area, exptime, scale;
+  wxString filter;
+  wxString photsys, fits_key_area, fits_key_exptime, fits_key_filter;
   FitsPhotosystems phsystems;
   std::vector<PhotoConv> phconv;
 
-  bool init(const std::vector<FitsArray>&);
-  void init_phsystem(const wxString&);
+  bool Init();
+  void Init_phsystem(const wxString&);
+  wxString ToString(double,int) const;
   double GetKeyDouble(const FitsHdu&, const wxString&) const;
 
 };
@@ -1122,9 +1118,7 @@ private:
 class FitsTime
 {
 public:
-  FitsTime():year(0), month(0), day(0), hour(0), minute(0), second(0),
-	     milisecond(0) {}
-  FitsTime(const wxString& );
+  FitsTime(const wxString&);
 
   void SetDateTime(const wxString& );
   void SetDate(const wxString& );
@@ -1146,6 +1140,8 @@ class FitsImage
 {
 public:
   FitsImage();
+  FitsImage(int, const long*);
+  FitsImage(const FitsImage&);
   FitsImage(const std::vector<FitsArray>&);
   FitsImage(const FitsArray&);
   FitsImage(const FitsArray&, const FitsArray&, const FitsArray&);
@@ -1154,18 +1150,18 @@ public:
 
   virtual FitsImage Thumb(int,int);
   virtual FitsImage GetSubImage(int,int,int,int);
-
-  virtual FitsImage Scale(float);
-  virtual FitsImage Scale(int,int);
-  virtual void Rescale(float);
   virtual void SetSubImage(int,int,const FitsImage&);
 
+  virtual FitsImage Shrink(int);
+  virtual FitsImage Scale(int,int);
+
   size_t GetCount() const;
   FitsArray Item(size_t) const;
   bool IsOk() const;
   bool IsColor() const;
   int GetWidth() const;
   int GetHeight() const;
+  long Npixels() const;
   const std::vector<FitsArray> GetArrays() const;
 
 private:
@@ -1180,6 +1176,8 @@ class FitsBitmap: public wxObject
 {
 public:
   FitsBitmap();
+  FitsBitmap(const FitsBitmap&);
+  FitsBitmap& operator = (const FitsBitmap&);
   FitsBitmap(int,int,unsigned char *);
   virtual ~FitsBitmap();
 
@@ -1193,73 +1191,185 @@ public:
 };
 
 
+class FitsBaseDisplay
+{
+ protected:
+  FitsBitmap bitmap;
+  FitsTone tone;
+  FitsItt itt;
+  int shrink;
+  long flags;
 
+  inline unsigned char fclip255(float) const;
+  inline float sRGBGamma(float) const;
+  inline float AdobeGamma(float) const;
+
+ public:
+  FitsBaseDisplay();
+  virtual ~FitsBaseDisplay() {};
+  virtual void SetArray(const FitsArray&) = 0;
+  virtual void SetImage(const FitsImage&) = 0;
+  void SetShrink(int);
+  void SetTone(const FitsTone&);
+  void SetItt(const FitsItt& );
+  virtual void SetColour(const FitsColor& ) {}
+  virtual void SetPalette(const FitsPalette&) {}
+
+  virtual void SetBlack(double);
+  virtual void SetSense(double);
+  virtual void SetQblack(double);
+  virtual void SetRsense(double);
+  virtual void SetAmp(double);
+  virtual void SetZero(double);
+  virtual void SetSaturation(double) {}
+  virtual void SetHue(double) {}
+  virtual void SetNiteThresh(double) {}
+  virtual void SetNiteWidth(double) {}
+  virtual void SetLevel(int,double) {}
+  virtual void SetPalette(const wxString&) {}
+  virtual void SetInversePalette(bool) {}
+  virtual void SetNiteVision(bool) {}
+  virtual void SetItt(const wxString&);
+  virtual void SetOperations(long);
+
+  virtual int GetWidth() const = 0;
+  virtual int GetHeight() const = 0;
+  virtual FitsBitmap GetBitmap() const { return bitmap; }
+  virtual FitsImage GetImage() const = 0;
+  virtual FitsImage GetShrinked() const = 0;
+  virtual void Render() = 0;
+
+};
+
+inline unsigned char FitsBaseDisplay::fclip255(float f) const
+{
+  return std::min(std::max(int(f+0.5),0),255);
+}
+
+inline float FitsBaseDisplay::sRGBGamma(float r) const
+{
+  if( r < 0.0031308f )
+    return 12.92f*r;
+  else
+    return 1.055f*powf(r,0.4166667f) - 0.055f;
+}
+
+inline float FitsBaseDisplay::AdobeGamma(float r) const
+{
+  const float q = 1.0/2.19921875;
+  return powf(r,q);
+}
+
+
+class FitsGrayDisplay: public FitsBaseDisplay
+{
+  FitsArray original, scaled, zoomed, current;
+  FitsPalette pal;
+
+  float *sRGBGamma(long, const float *) const;
+  float *AdobeGamma(long, const float *) const;
+
+ public:
+  FitsGrayDisplay(const FitsArray&);
+  FitsGrayDisplay(const FitsImage&);
+  void SetArray(const FitsArray&);
+  void SetImage(const FitsImage&);
+  void SetPalette(const FitsPalette&);
+  void SetPalette(const wxString& p);
+  void SetInversePalette(bool);
+  void SetColourspace(const wxString&);
+  int GetWidth() const;
+  int GetHeight() const;
+  FitsImage GetImage() const;
+  FitsImage GetShrinked() const { return zoomed; }
+  void Render();
+
+};
+
+class FitsColourDisplay: public FitsBaseDisplay
+{
+  std::vector<FitsArray> originals, scaled, zoomed;
+  FitsColor colour;
+
+  unsigned char *XYZ_sRGB(long,
+			  const float *, const float *, const float *) const;
+  unsigned char *XYZ_AdobeRGB(long,
+			      const float *,const float *,const float *) const;
+
+ public:
+  FitsColourDisplay(const std::vector<FitsArray>&);
+  FitsColourDisplay(const FitsArray&);
+  FitsColourDisplay(const FitsImage&);
+  void SetArray(const FitsArray&);
+  void SetImage(const FitsImage&);
+  void SetColour(const FitsColor& );
+  void SetSaturation(double);
+  void SetHue(double);
+  void SetNiteThresh(double);
+  void SetNiteWidth(double);
+  void SetLevel(int,double);
+  void SetNiteVision(bool);
+  void SetColourspace(const wxString&);
+  int GetWidth() const;
+  int GetHeight() const;
+  FitsImage GetImage() const;
+  FitsImage GetShrinked() const;
+  void Render();
+
+};
 
 class FitsDisplay
 {
 public:
-  FitsDisplay(const wxString& =wxEmptyString);
+  FitsDisplay(const FitsArray&);
+  FitsDisplay(const FitsImage&);
+  FitsDisplay(const FitsDisplay&);
+  FitsDisplay& operator = (const FitsDisplay&);
+  virtual ~FitsDisplay();
 
   virtual void SetPalette(const FitsPalette&);
+  virtual void SetTone(const FitsTone&);
   virtual void SetItt(const FitsItt& );
   virtual void SetColor(const FitsColor& );
-  virtual void SetTemperature(float);
-  virtual void SetColorspace(const wxString&);
+  virtual void SetShrink(int);
+  virtual void SetShrink(int,int);
+  virtual void SetOperations(long);
 
+  FitsTone GetTone() const;
   FitsItt GetItt() const;
   FitsPalette GetPalette() const;
   FitsColor GetColor() const;
-  float GetTemperature() const;
-  wxString GetColorspace() const;
-  wxArrayString GetArraySpaces() const;
 
-  FitsBitmap GetImage(const FitsImage&);
+  FitsBitmap GetImage() const;
 
 private:
 
-  FitsBitmap ConvertOfGray(const FitsArray&);
-  //  FitsBitmap ConvertOfColor(const FitsArray&,const FitsArray&,const FitsArray&);
-  FitsBitmap ConvertOfColor(const std::vector<FitsArray>&);
-  FitsBitmap ConvertOfColorX(const std::vector<FitsArray>&);
-
-  unsigned char *XYZ_sRGB(size_t, const float *, const float *, const float *);
-  unsigned char *XYZ_AdobeRGB(size_t,const float *,const float *,const float *);
-
-  inline unsigned char Cutoff(float) const;
-  inline float Gamma(float);
-  float *sRGBGamma(int,float *);
-  float *AdobeGamma(int,float *);
-  unsigned char *Crange(size_t,const float *);
-
-  wxString cspace;
-  wxArrayString spaces;
+  FitsBaseDisplay *display;
+  FitsTone tone;
   FitsItt itt;
   FitsPalette pal;
   FitsColor color;
-  float temp,xtemp,ytemp,ztemp;
 };
 
 
-
-inline unsigned char FitsDisplay::Cutoff(float f) const
+class FitsZoom
 {
-  int j = int(f);
+  int shrink,zoom;
+  static const int nratios;
+  static const int ratios[];
 
-  if( j < 0 )
-    return 0;
-  else if( j > 255 )
-    return 255;
-  else
-    return (unsigned char) j;
-}
+public:
+  FitsZoom();
+  FitsZoom(int,int=ZOOM_SHRINK);
+  FitsZoom(const FitsZoom&);
+  FitsZoom& operator = (const FitsZoom&);
 
-inline float FitsDisplay::Gamma(float r)
-{
-  if( r < 0.00304 )
-    return 12.92*r;
-  else
-    return 1.055*powf(r,0.4166667) - 0.055;
-}
+  int GetKind() const;
+  int GetZoom() const;
+  int GetShrink() const;
+  FitsZoom GetIn() const;
+  FitsZoom GetOut() const;
+};
 
 
 // auxiliary  functions
