@@ -1,7 +1,7 @@
 /*
    xmunipack - coordinates
 
-   Copyright © 1997-2013, 2017 F.Hroch (hroch@physics.muni.cz)
+   Copyright © 1997-2013, 2017, 2019 F.Hroch (hroch@physics.muni.cz)
 
    This file is part of Munipack.
 
@@ -189,60 +189,10 @@ void FitsCoo::DecSix(double delta, char& sign, int& d, int& m, double& s)
   s = 3600.0*(y - (d + m/60.0));
 }
 
-
-wxString FitsCoo::Get_str(int i, int j)
-{
-  wxString a,b;
-  GetStr(i,j,a,b);
-
-  if( a.IsEmpty() || a.IsEmpty() )
-    return wxEmptyString;
-
-  if( type == COO_PIXEL )
-    return "x=" + a + "  y=" + b;
-  else
-    return L"α=" + a + L"  δ=" + b;
-}
-
-
-void FitsCoo::GetStr(int i, int j, wxString& a, wxString& b)
-{
-  int h,min,d,m;
-  double x,y, sec,s;
-  char sg;
-
-  a.Clear();
-  b.Clear();
-
-  if( 0 <= i && i < Width() && 0 <= j && j < Height() ) {
-
-    if( type == COO_PIXEL ) {
-      a.Printf("%d",i+1);
-      b.Printf("%d",j+1);
-    }
-    else if( haswcs ) {
-
-      GetEq(i,j,x,y);
-
-      if( type == COO_EQDEG ) {
-	a.Printf(L"%.5f°",x);
-	b.Printf(L"%.5f°",y);
-      }
-
-      else if( type == COO_EQSIX ) {
-	RaSix(x,h,min,sec);
-	DecSix(y,sg,d,m,s);
-	a.Printf(L"%02d:%02d:%06.3fs",h,min,sec);
-	b.Printf(L"%c%02d°%02d'%05.2f\"",sg,d,m,s);
-      }
-    }
-  }
-}
-
 void FitsCoo::GetPix(int i, int j, wxString& a, wxString& b)
 {
   if( 0 <= i && i < Width() && 0 <= j && j < Height() ) {
-    a.Printf("%d",i+1);
+    a.Printf("%d",i+1); // C-style indexing should be considered very funny!
     b.Printf("%d",j+1);
   }
   else {
