@@ -3,7 +3,7 @@
   xmunipack - FITS view
 
 
-  Copyright © 2009 - 2014, 2017 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2009 - 2014, 2017-9 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -23,6 +23,7 @@
 */
 
 #include "xmunipack.h"
+#include "structtree.h"
 #include "tune.h"
 #include <wx/wx.h>
 #include <wx/toolbar.h>
@@ -32,12 +33,12 @@
 #include <wx/wupdlock.h>
 #include <wx/combo.h>
 #include <wx/treectrl.h>
-#include <wx/animate.h>
+#include <wx/progdlg.h>
 #include <cfloat>
 #include <list>
 
 #define TONE_CURVES "Tone curves"
-#define COLOR_PALETTES "Color Palettes"
+#define COLOR_PALETTES "Colour palettes"
 #define MENU_FRAME "Frame"
 #define MENU_HEAD "Head"
 #define MENU_GRID "Table"
@@ -49,287 +50,11 @@
 using namespace std;
 
 
-// ------------------------------------------------------------------------
-
-class MuniSplashing: public wxWindow
-{
-private:
-
-  wxBitmap logo;
-  wxAnimationCtrl *anim;
-
-public:
-  MuniSplashing(wxWindow *w, const MuniConfig *config):
-    wxWindow(w,wxID_ANY),logo(wxBitmap(config->munipack_icon)),anim(0)
-  {
-    //    wxLogDebug("Splashing...");
-
-    wxFont bf(*wxNORMAL_FONT);
-    bf.SetWeight(wxFONTWEIGHT_BOLD);
-    bf.Scale(2.0);
-
-    anim = new wxAnimationCtrl(this,wxID_ANY,config->throbber);
-    wxBoxSizer *topsizer = new wxBoxSizer(wxVERTICAL);
-    topsizer->AddStretchSpacer();
-    topsizer->Add(new wxStaticBitmap(this,wxID_ANY,logo),wxSizerFlags().Center().Border());
-    wxStaticText *label = new wxStaticText(this,wxID_ANY,"Munipack");
-    label->SetFont(bf);
-    label->SetForegroundColour(wxColour(128,128,128));
-    topsizer->Add(label,wxSizerFlags().Center().DoubleBorder());
-    topsizer->Add(anim,wxSizerFlags().Center().Border());
-    topsizer->AddStretchSpacer();
-    SetSizer(topsizer);
-
-    //    anim->Play();
-  }
-
-  void Play() { anim->Play(); }
-  void Stop() { anim->Stop(); }
-
-};
-
-
-// ------------------------------------------------------------------------
-
-class xTreeItemData: public wxTreeItemData
-{
-public:
-  xTreeItemData(const int n): index(n) {}
-
-  int index;
-};
-
-
-// ------------------------------------------------------------
-
-
-class MuniStructtree: public wxTreeCtrl, public wxComboPopup
-{
-private:
-
-  // an initial code fork from wxWidgets docs, chapter:
-  //      Setting Custom Popup for wxComboCtrl
-
-protected:
-
-    wxTreeItemId        m_value; // current item index
-    wxTreeItemId        m_itemHere; // hot item in popup
-
-  virtual void Init() {
-
-    Bind(wxEVT_MOTION,&MuniStructtree::OnMouseMove,this);
-    Bind(wxEVT_LEFT_DOWN,&MuniStructtree::OnMouseClick,this);
-
-  }
-
-  // Create popup control
-  virtual bool Create(wxWindow* parent) {
-
-    return wxTreeCtrl::Create(parent,wxID_ANY,wxDefaultPosition,wxDefaultSize,
-			      (wxTR_DEFAULT_STYLE | wxTR_HAS_BUTTONS | wxTR_HIDE_ROOT | wxTR_FULL_ROW_HIGHLIGHT) & ~wxTR_NO_LINES);
-
-  }
-
-
-  // Return pointer to the created control
-  virtual wxWindow *GetControl() { return this; }
-
-
-    virtual wxSize GetAdjustedSize( int minWidth,
-                                    int WXUNUSED(prefHeight),
-                                    int maxHeight )
-    {
-        return wxSize(wxMax(300,minWidth),wxMin(250,maxHeight));
-    }
-
-    // Needed by SetStringValue
-    wxTreeItemId FindItemByText( wxTreeItemId parent, const wxString& text )
-    {
-        wxTreeItemIdValue cookie;
-        wxTreeItemId child = GetFirstChild(parent,cookie);
-        while ( child.IsOk() )
-        {
-            if ( GetItemText(child) == text )
-            {
-                return child;
-            }
-            if ( ItemHasChildren(child) )
-            {
-                wxTreeItemId found = FindItemByText(child,text);
-                if ( found.IsOk() )
-                    return found;
-            }
-            child = GetNextChild(parent,cookie);
-        }
-        return wxTreeItemId();
-    }
-
-    virtual void SetStringValue( const wxString& s )
-    {
-        wxTreeItemId root = GetRootItem();
-        if ( !root.IsOk() )
-            return;
-
-        wxTreeItemId found = FindItemByText(root,s);
-        if ( found.IsOk() )
-        {
-            m_value = m_itemHere = found;
-            wxTreeCtrl::SelectItem(found);
-        }
-    }
-
-    virtual wxString GetStringValue() const
-    {
-        if ( m_value.IsOk() )
-            return wxTreeCtrl::GetItemText(m_value);
-        return wxEmptyString;
-    }
-
-    //
-    // Popup event handlers
-    //
-
-    // Mouse hot-tracking
-    void OnMouseMove(wxMouseEvent& event)
-    {
-        int resFlags;
-        wxTreeItemId itemHere = HitTest(event.GetPosition(),resFlags);
-        if ( itemHere.IsOk() &&
-	     ((resFlags & wxTREE_HITTEST_ONITEMLABEL) ||
-	      (resFlags & wxTREE_HITTEST_ONITEMICON) ||
-	      (resFlags & wxTREE_HITTEST_ONITEMSTATEICON) ||
-	      (resFlags & wxTREE_HITTEST_ONITEMBUTTON) ||
-	      (resFlags & wxTREE_HITTEST_ONITEMINDENT) ||
-	      (resFlags & wxTREE_HITTEST_ONITEMRIGHT) ) )
-        {
-            wxTreeCtrl::SelectItem(itemHere,true);
-            m_itemHere = itemHere;
-        }
-        event.Skip();
-    }
-
-    // On mouse left, set the value and close the popup
-    void OnMouseClick(wxMouseEvent& event)
-    {
-        int resFlags;
-        wxTreeItemId itemHere = HitTest(event.GetPosition(),resFlags);
-        if ( itemHere.IsOk() &&
-	     ((resFlags & wxTREE_HITTEST_ONITEMLABEL) ||
-	      (resFlags & wxTREE_HITTEST_ONITEMICON) ||
-	      (resFlags & wxTREE_HITTEST_ONITEMSTATEICON) ||
-	      (resFlags & wxTREE_HITTEST_ONITEMBUTTON) ||
-	      (resFlags & wxTREE_HITTEST_ONITEMINDENT) ||
-	      (resFlags & wxTREE_HITTEST_ONITEMRIGHT) ) )
-        {
-            m_itemHere = itemHere;
-            m_value = itemHere;
-            Dismiss();
-
-	    wxTreeItemId item = GetFocusedItem();
-	    if( item.IsOk() ) {
-	      xTreeItemData *data = dynamic_cast<xTreeItemData *>(GetItemData(item));
-	      wxASSERT(data);
-
-	      MuniHduEvent ev(EVT_HDU);
-	      ev.hdu = data->index;
-	      wxQueueEvent(GetGrandParent(),ev.Clone());
-	    }
-
-        }
-        event.Skip();
-    }
-
-  void PaintComboControl(wxDC& dc, const wxRect& rect)
-  {
-    wxTreeItemId item = GetFocusedItem();
-    if( item.IsOk() ) {
-
-      wxString label(GetItemText(item));
-
-      wxImageList *ilist = GetImageList();
-      int index = GetItemImage(item);
-      if( index > -1 ) {
-	wxBitmap icon(ilist->GetBitmap(index));
-	dc.DrawLabel(label,icon,rect,wxALIGN_CENTER|wxALIGN_CENTER_VERTICAL);
-      }
-      else
-	dc.DrawLabel(label,rect,wxALIGN_CENTER|wxALIGN_CENTER_VERTICAL);
-    }
-    else
-      dc.DrawLabel("-",rect,wxALIGN_CENTER|wxALIGN_CENTER_VERTICAL);
-
-  }
-
-public:
-
-  void SetMeta(const FitsMeta& meta) {
-
-    DeleteAllItems();
-
-    wxFont fn(*wxNORMAL_FONT);
-    int ssize = 2*(fn.GetPointSize()+3);
-    wxImageList *icons = new wxImageList(ssize, ssize, true);
-    for(size_t n = 0; n < meta.HduCount(); n++ ) {
-      const FitsMetaHdu hdu = meta.Hdu(n);
-      icons->Add(wxBitmap(MuniIcon::ListIcon(hdu.GetIcon(),ssize)));
-    }
-    icons->Add(wxBitmap(MuniIcon::ListIcon(meta.GetIcon(),ssize)));
-    AssignImageList(icons);
-
-    wxTreeItemId root = AddRoot("<root>");
-
-    wxTreeItemId hub = AppendItem(root,meta.GetName(),meta.HduCount(),-1,new xTreeItemData(-1));
-    for(size_t n = 0; n < meta.HduCount(); n++ ) {
-      const FitsMetaHdu hdu = meta.Hdu(n);
-      wxString label = hdu.GetControlLabel();
-      AppendItem(hub,label,n,-1,new xTreeItemData(n));
-    }
-    Expand(hub);
-  }
-
-  void SetHdu(int hdu)
-  {
-    wxTreeItemId root = GetRootItem();
-
-    wxTreeItemIdValue cookie;
-    wxTreeItemId hub = GetFirstChild(root,cookie);
-    while ( hub.IsOk() ) {
-
-      xTreeItemData *data = dynamic_cast<xTreeItemData *>(GetItemData(hub));
-      wxASSERT(data);
-      if( data->index == hdu ) {
-	SelectItem(hub);
-	return;
-      }
-      else {
-
-	wxTreeItemIdValue cookies;
-	wxTreeItemId child = GetFirstChild(hub,cookies);
-	while ( child.IsOk() ) {
-
-	  xTreeItemData *data = dynamic_cast<xTreeItemData *>(GetItemData(child));
-	  wxASSERT(data);
-	  if( data->index == hdu ) {
-	    SelectItem(child);
-	    return;
-	  }
-
-	  child = GetNextChild(hub,cookies);
-        }
-      }
-      hub = GetNextChild(root,cookie);
-    }
-  }
-};
-
-
-// ------------------------------------------------------------------------
-
-
 MuniView::MuniView(wxWindow *w, MuniConfig *c):
-  wxFrame(w, wxID_ANY,"untitled", wxDefaultPosition, c->view_size), loader(0),
-  config(c), hdusel(0), /*ntools(0),*/ place(0), onreset(false),
-  coloring(0),
+  wxFrame(w, wxID_ANY,"untitled", wxDefaultPosition, c->view_size),
+  config(c), hdusel(0),
+  palinv(config->display_palinv), palauto(config->display_palauto),
+  loader(0), zoomctrl(0), place(0), coloring(0),
   console(new wxLogWindow(this,"Logs",false))
   /*shell(0),*/
 {
@@ -343,7 +68,7 @@ MuniView::MuniView(wxWindow *w, MuniConfig *c):
   //  menuFile->Append(ID_NEW_BROWSER,_("&New Browser..."));
   menuFile->Append(wxID_OPEN);
   wxMenu *menuFileCreate = new wxMenu;
-  menuFileCreate->Append(ID_COLORING,"Color Image...");
+  menuFileCreate->Append(ID_COLORING,"Colour Image...");
   //  menuFileCreate->Append(wxID_ANY,"Image server ...");
   //  menuFileCreate->Append(wxID_ANY,"Synthetic image...");
   menuFile->AppendSubMenu(menuFileCreate,"Create");
@@ -413,26 +138,19 @@ MuniView::MuniView(wxWindow *w, MuniConfig *c):
   MuniArtIcons ico(wxART_TOOLBAR,wxSize(22,22));
 
   wxToolBar *tbar = CreateToolBar(wxTB_HORIZONTAL|wxTB_TEXT);
-  tbar->AddTool(wxID_BACKWARD,"Previous",ico.Icon(wxART_GO_BACK),"Go to previous HDU");
-  tbar->AddTool(wxID_FORWARD,"Next",ico.Icon(wxART_GO_FORWARD),"Go to next HDU");
-  tbar->AddTool(wxID_HOME,"Structure",ico.Icon(wxART_GO_HOME),"Show structure of this FITS file");
+  tbar->AddTool(ID_INFO,"Header",ico.Icon(wxART_INFORMATION),
+		"Show FITS header");
+  tbar->AddSeparator();
+  tbar->AddStretchableSpace();
+
   structtree = new wxComboCtrl(tbar,wxID_ANY,wxEmptyString,wxDefaultPosition,
 			       wxDefaultSize,wxCB_READONLY|wxCC_STD_BUTTON);
   MuniStructtree* popupCtrl = new MuniStructtree;
   structtree->SetPopupControl(popupCtrl);
 
   tbar->AddControl(structtree);
-  tbar->AddTool(ID_INFO,"Header",ico.Icon(wxART_INFORMATION),"Show FITS header");
-  tbar->AddSeparator();
-
-  tbar->AddStretchableSpace();
-
 
   tbar->EnableTool(ID_INFO,false);
-  tbar->EnableTool(wxID_HOME,false);
-  tbar->EnableTool(wxID_BACKWARD,false);
-  tbar->EnableTool(wxID_FORWARD,false);
-
 
   tbar->Realize();
   tbar->Show(config->view_tbar);
@@ -464,7 +182,6 @@ MuniView::MuniView(wxWindow *w, MuniConfig *c):
   Bind(wxEVT_CLOSE_WINDOW,&MuniView::OnClose,this);
   //  Bind(wxEVT_SIZE,&MuniView::OnSize,this);
   //  Bind(wxEVT_ENTER_WINDOW,&MuniView::OnEnterWin,this);
-  //  Connect(wxEVT_LEAVE_WINDOW,wxMouseEventHandler(MuniView::OnLeaveWin));
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::FileClose,this,wxID_CLOSE);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::FileClose,this,wxID_EXIT);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::NewView,this,wxID_NEW);
@@ -472,7 +189,6 @@ MuniView::MuniView(wxWindow *w, MuniConfig *c):
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::FileSave,this,wxID_SAVE);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::FileExport,this,ID_EXPORT);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::FileProperties,this,wxID_PROPERTIES);
-  //  Connect(wxID_PREFERENCES,wxEVT_COMMAND_MENU_SELECTED,
   //	  wxCommandEventHandler(MuniView::OnPreferences));
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::ShowToolbar,this,ID_TOOLBAR);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnFullScreen,this,ID_FULLSCREEN);
@@ -484,15 +200,9 @@ MuniView::MuniView(wxWindow *w, MuniConfig *c):
   //  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::HelpBug,this,ID_BUG);
 
   Bind(EVT_HDU,&MuniView::OnResumeHdu,this);
-  Bind(EVT_CONFIG_UPDATED,&MuniView::OnConfigUpdated,this);
-  //  Connect(EVT_CONFIG_UPDATED,wxCommandEventHandler(MuniView::OnConfigUpdated));
-  //  Bind(EVT_FILELOAD,&MuniView::OnFileLoad,this);
 
 }
 
-MuniView::~MuniView() {}
-
-
 void MuniView::OnClose(wxCloseEvent& event)
 {
   if( event.CanVeto() && IsModified() ) {
@@ -517,7 +227,6 @@ void MuniView::OnClose(wxCloseEvent& event)
   // probbaly crashes during loading (see close functions in astrometry)
   StopLoading();
 
-  //  if( display->IsRendering() )
   MuniDisplay *display = dynamic_cast<MuniDisplay *>(place);
   if( display )
     display->StopRendering();
@@ -532,12 +241,14 @@ void MuniView::OnClose(wxCloseEvent& event)
   if( GetParent() )
     wxQueueEvent(GetParent(),event.Clone());
 
+  config->display_palinv = palinv;
+  config->display_palauto = palauto;
   config->view_tbar = GetToolBar()->IsShown();
   config->view_size = GetSize();
 
+
   // see above
   Destroy();
-
 }
 
 bool MuniView::IsModified() const
@@ -554,7 +265,6 @@ void MuniView::RemoveBackup()
   backup.Clear();
 }
 
-
 void MuniView::ViewLog(wxCommandEvent& event)
 {
   wxASSERT(console);
@@ -562,26 +272,6 @@ void MuniView::ViewLog(wxCommandEvent& event)
 }
 
 
-void MuniView::OnUpdateFit(wxUpdateUIEvent& event)
-{
-  wxTopLevelWindow *twin = static_cast<wxTopLevelWindow *>(this);
-  if( !twin || twin->IsFullScreen()) return;
-
-  MuniDisplay *display = dynamic_cast<MuniDisplay *>(place);
-  if( display )
-    event.Check(display->GetFit());
-}
-
-void MuniView::OnUpdateNFit(wxUpdateUIEvent& event)
-{
-  wxTopLevelWindow *twin = static_cast<wxTopLevelWindow *>(this);
-  if( !twin || twin->IsFullScreen()) return;
-
-  MuniDisplay *display = dynamic_cast<MuniDisplay *>(place);
-  if( display )
-    event.Enable(!display->GetFit());
-}
-
 void MuniView::OnUpdateInfos(wxUpdateUIEvent& event)
 {
   event.Check(GetToolBar()->GetToolState(ID_INFO));
@@ -641,19 +331,11 @@ void MuniView::LoadFileBackup(const wxString& file, const wxString& b)
   //  LoadStart(fits.GetFullPath());
 }
 
-/*
-void MuniView::ReloadFile()
-{
-  if( fits.Status() ) {
-    LoadStart(backup.Last());
-  }
-}
-*/
-
 int MuniView::Unsaved(const wxString& filename)
 {
-  wxMessageDialog dialog(this,"Do you want to save changes you made in \""+filename+"\" ?",
-			 filename,wxICON_EXCLAMATION | wxYES_NO | wxCANCEL | wxNO_DEFAULT);
+  wxMessageDialog dialog(this,"Do you want to save changes you made in \""+
+			 filename+"\" ?",filename,wxICON_EXCLAMATION |
+			 wxYES_NO | wxCANCEL | wxNO_DEFAULT);
   dialog.SetExtendedMessage("Your changes will be lost if you don't save them.");
   dialog.SetYesNoLabels("Save","Don't Save");
 
@@ -683,15 +365,12 @@ void MuniView::LoadStart(const wxString& filename)
 
   Bind(EVT_FITS_OPEN,&MuniView::OnLoadFinish,this,ID_LOADER);
 
-  loader = new FitsOpen(this,filename);
+  loader = new FitsOpen(this,filename,config->icon_size);
   wxASSERT(loader);
   wxThreadError code = loader->Create();
   wxASSERT(code == wxTHREAD_NO_ERROR);
   loader->Run();
 
-  //  ::wxBeginBusyCursor();
-
-  //  Clear(); // ???
   if( dynamic_cast<MuniSplashing *>(place) )
     dynamic_cast<MuniSplashing *>(place)->Play();
 
@@ -701,8 +380,6 @@ void MuniView::OnLoadFinish(FitsOpenEvent& event)
 {
   //  wxLogDebug("MuniView::OnLoadFinish");
 
-  //  wxString oldname = fits.GetName();
-
   // wait for thread finish
   while(true) {
     {
@@ -712,21 +389,22 @@ void MuniView::OnLoadFinish(FitsOpenEvent& event)
     ::wxMilliSleep(1);
   }
 
+  if( dynamic_cast<MuniSplashing *>(place) )
+    dynamic_cast<MuniSplashing *>(place)->Stop();
 
   fits = event.fits;
+  vector<wxImage> icons(event.icons);
 
   if( fits.Status() ) {
 
     if( ! meta.IsOk() ) {
       MuniIcon micon(fits,config->display_colorspace,config->cdatafile,
 		     config->default_icon,config->table_icon,config->head_icon,
-		     config->icon_size);
+		     config->icon_size,icons);
       meta = FitsMeta(fits,micon.GetIcon(),micon.GetList());
       SetMeta(meta);
     }
-
-    wxASSERT(meta.IsOk());
-    wxASSERT(fits.HduCount() > 0);
+    wxASSERT(meta.IsOk() && fits.HduCount() > 0);
 
     hdusel = EstimHduSel(meta);
     HduSelect(hdusel);
@@ -745,14 +423,21 @@ void MuniView::OnLoadFinish(FitsOpenEvent& event)
   }
 
   Unbind(EVT_FITS_OPEN,&MuniView::OnLoadFinish,this,ID_LOADER);
-
-  if( dynamic_cast<MuniSplashing *>(place) )
-    dynamic_cast<MuniSplashing *>(place)->Stop();
-
-  //  ::wxEndBusyCursor();
 }
 
+void MuniView::ExportStart(const wxString& savename)
+{
+  loader = new FitsExport(this,fits.Hdu(hdusel),savename,
+			  dynamic_cast<MuniDisplay *>(place)->GetTone(),
+			  dynamic_cast<MuniDisplay *>(place)->GetItt(),
+			  dynamic_cast<MuniDisplay *>(place)->GetPalette(),
+			  dynamic_cast<MuniDisplay *>(place)->GetColor());
+  wxASSERT(loader);
+  wxThreadError code = loader->Create();
+  wxASSERT(code == wxTHREAD_NO_ERROR);
+  loader->Run();
 
+}
 
 
 void MuniView::Clear()
@@ -978,17 +663,20 @@ void MuniView::FileExport(wxCommandEvent& WXUNUSED(event))
 			"PNG files (*.png)|*.png|JPEG files (*.jpg)|*.jpg|TIFF files (*.tif)|*.tif|PNM files (*.pnm)|*.pnm",
 			wxFD_SAVE|wxFD_OVERWRITE_PROMPT|wxFD_CHANGE_DIR);
 
-    if (select.ShowModal() == wxID_OK ) {
+    if (select.ShowModal() == wxID_CANCEL )
+      return;
 
-      /*
-      wxProgressDialog dialog(_("Export of Image"),_("Exporting ")+
-			      select.GetFilename()+_(" ... "),100,this,
-			      wxPD_APP_MODAL|wxPD_AUTO_HIDE);
+    // save thread
+    ExportStart(select.GetPath());
+
+    wxProgressDialog dialog("Export of an image","Exporting "+
+			    select.GetFilename()+" ... ",100,this,
+			    wxPD_APP_MODAL|wxPD_AUTO_HIDE);
+    while(true) {
       dialog.Pulse();
-      */
-      wxBusyCursor wait;
-      wxImage image = dynamic_cast<MuniDisplay *>(place)->GetImage();
-      image.SaveFile(select.GetPath());
+      ::wxMilliSleep(50);
+      wxCriticalSectionLocker enter(loaderCS);
+      if( ! loader ) break;
     }
   }
 
@@ -1082,13 +770,6 @@ void MuniView::FileProperties(wxCommandEvent& WXUNUSED(event))
   w->Show();
 }
 
-// void MuniView::OnFileLoad(wxCommandEvent& event)
-// {
-//   //  wxLogDebug(event.GetString());
-//   LoadFile(event.GetString());
-// }
-
-
 void MuniView::OnPreferences(wxCommandEvent& WXUNUSED(event))
 {
   MuniPreferences *w = new MuniPreferences(this,config);
@@ -1104,87 +785,112 @@ void MuniView::OnSelectFrame(wxCommandEvent& event)
 void MuniView::OnCycleZoom(wxCommandEvent& event)
 {
   wxASSERT(dynamic_cast<MuniDisplay *>(place));
+  wxASSERT(zoomctrl);
 
-  double z = dynamic_cast<MuniDisplay *>(place)->GetZoom();
-  double zoom = -1.0;
-
-  switch(event.GetId()) {
-  case wxID_ZOOM_IN: zoom = FitsZoom::FindGreater(z); break;
-  case wxID_ZOOM_OUT: zoom = FitsZoom::FindSmaller(z); break;
-  }
+  FitsZoom zoom = dynamic_cast<MuniDisplay *>(place)->GetZoom();
 
   MuniTuneEvent ev(EVT_TUNE,ID_ZOOM_SCALE);
-  ev.x = zoom;
-  if( event.GetId() == wxID_ZOOM_FIT )
-    ev.fit = event.IsChecked();
-  else
-    ev.fit = false;
 
+  if( event.GetId() == wxID_ZOOM_FIT ) {
+    ev.shrink = true;
+    ev.n = dynamic_cast<MuniDisplay *>(place)->GetFitShrink();
+  }
+  else if ( event.GetId() == wxID_ZOOM_100 ) {
+    ev.shrink = true;
+    ev.n = 1;
+  }
+  else {
+    //wxLogDebug("MuniView::OnCycleZoom %d %d %d",zoom.GetShrink(),
+    //	       zoom.GetZoom(),event.GetId()==wxID_ZOOM_IN);
+    FitsZoom z;
+    switch(event.GetId()) {
+    case wxID_ZOOM_IN:  z = zoom.GetIn(); break;
+    case wxID_ZOOM_OUT: z = zoom.GetOut(); break;
+    }
+
+
+    ev.shrink = z.GetKind() == ZOOM_SHRINK;
+    if(  ev.shrink )
+      ev.n = z.GetShrink();
+    else
+      ev.n = z.GetZoom();
+  }
   wxQueueEvent(place,ev.Clone());
+
+  wxASSERT(zoomctrl);
+  wxString a;
+  if( ev.shrink )
+    a.Printf("1:%d",ev.n);
+  else
+    a.Printf("%d:1",ev.n);
+  zoomctrl->SetValue(a);
 }
 
 void MuniView::OnMenuZoom(wxCommandEvent& event)
 {
-  wxASSERT(place);
-
-  double zoom = 1.0;
-
-  wxString entry;
-  switch(event.GetId()) {
-  case wxID_ZOOM_FIT: zoom = -1.0; break;
-  case wxID_ZOOM_100: zoom = 1.0; break;
-  }
+  wxASSERT(place && zoomctrl);
 
   MuniTuneEvent ev(EVT_TUNE,ID_ZOOM_SCALE);
-  ev.x = zoom;
-  if( event.GetId() == wxID_ZOOM_FIT )
-    ev.fit = event.IsChecked();
+  ev.shrink = true;
+
+  if( event.GetId() == wxID_ZOOM_100 )
+    ev.n = 1;
+  else if( event.GetId() == wxID_ZOOM_FIT )
+    ev.n = dynamic_cast<MuniDisplay *>(place)->GetFitShrink();
+
+  wxQueueEvent(place,ev.Clone());
+
+  wxString a;
+  if( ev.shrink )
+    a.Printf("1:%d",ev.n);
   else
-    ev.fit = false;
-
-  wxQueueEvent(place,ev.Clone());
+    a.Printf("%d:1",ev.n);
+  zoomctrl->SetValue(a);
 }
 
-void MuniView::OnMenuItt(wxCommandEvent& event)
+
+void MuniView::OnToolZoom(wxCommandEvent& event)
 {
-  wxMenu *menuTune = GetMenuBar()->GetMenu(GetMenuBar()->FindMenu(MENU_TUNE));
-  wxMenuItem *item = menuTune->FindItem(menuTune->FindItem(TONE_CURVES));
-  wxMenu *menuProfile= item->GetSubMenu();
-  wxASSERT(place && menuTune && menuProfile &&
-	   menuProfile->FindItem(event.GetId()));
+  wxASSERT(place);
 
-  MuniTuneEvent ev(EVT_TUNE,ID_ITT_TYPE);
-  wxMenuItem *sitem = menuProfile->FindItem(event.GetId());
-  ev.SetString(sitem->GetItemLabelText());
-  wxQueueEvent(place,ev.Clone());
-}
+  MuniTuneEvent ev(EVT_TUNE,ID_ZOOM_SCALE);
 
-void MuniView::OnCycleItt(wxCommandEvent& event)
-{
-  wxMenu *menuTune = GetMenuBar()->GetMenu(GetMenuBar()->FindMenu(MENU_TUNE));
-  wxMenuItem *item = menuTune->FindItem(menuTune->FindItem(TONE_CURVES));
-  wxMenu *menuProfile= item->GetSubMenu();
-  wxASSERT(place && menuTune && menuProfile);
-
-  size_t nmenu = menuProfile->GetMenuItemCount();
-  wxString label;
-  int itemid = -1;
-  for(size_t i = 0; i < nmenu; i++) {
-    wxMenuItem *item = menuProfile->FindItemByPosition(i);
-    if( item && item->IsChecked() ) {
-      wxMenuItem *it = menuProfile->FindItemByPosition(i+1 < nmenu ? i+1 : 0);
-      if( it ) {
-	label = it->GetItemLabelText();
-	itemid = it->GetId();
-      }
-    }
+  if( event.GetString() == "4:1" ) {
+    ev.shrink = false;
+    ev.n = 4;
   }
-  wxASSERT(!label.IsEmpty());
+  else if( event.GetString() == "2:1" ) {
+    ev.n = 2;
+    ev.shrink = false;
+  }
+  else if( event.GetString() == "1:1" ) {
+    ev.n = 1;
+    ev.shrink = true;
+  }
+  else if( event.GetString() == "1:2" ) {
+    ev.n = 2;
+    ev.shrink = true;
+  }
+  else if( event.GetString() == "1:3" ) {
+    ev.n = 3;
+    ev.shrink = true;
+  }
+  else if( event.GetString() == "1:4" ) {
+    ev.n = 4;
+    ev.shrink = true;
+  }
+  else if( event.GetString() == "1:6" ) {
+    ev.n = 6;
+    ev.shrink = true;
+  }
+  else if( event.GetString() == "1:8" ) {
+    ev.n = 8;
+    ev.shrink = true;
+  }
+  else
+    wxFAIL_MSG("MuniView::OnToolZoom: UNKNOWN ZOOM");
 
-  menuProfile->Check(itemid,true);
-
-  MuniTuneEvent ev(EVT_TUNE,ID_ITT_TYPE);
-  ev.SetString(label);
+  //  wxLogDebug("MuniView::OnToolZoom: "+event.GetString());
   wxQueueEvent(place,ev.Clone());
 }
 
@@ -1231,15 +937,23 @@ void MuniView::OnCyclePal(wxCommandEvent& event)
   wxQueueEvent(place,ev.Clone());
 }
 
-void MuniView::OnMenuNegative(wxCommandEvent& event)
+void MuniView::OnMenuInverse(wxCommandEvent& event)
 {
   wxASSERT(place);
 
-  MuniTuneEvent ev(EVT_TUNE,ID_PALETTE_NEGATIVE);
+  palinv = event.IsChecked();
+
+  MuniTuneEvent ev(EVT_TUNE,ID_PALETTE_INVERSE);
   ev.SetInt(event.IsChecked());
   wxQueueEvent(place,ev.Clone());
 }
 
+void MuniView::OnMenuPalauto(wxCommandEvent& event)
+{
+  wxASSERT(place);
+  palauto = event.IsChecked();
+  wxQueueEvent(place,event.Clone());
+}
 
 
 void MuniView::OnTuneFine(MuniTuneEvent& event)
@@ -1251,18 +965,10 @@ void MuniView::OnTuneFine(MuniTuneEvent& event)
     menuPalette = item->GetSubMenu();
 
   item = menuTune->FindItem(menuTune->FindItem(TONE_CURVES));
-  wxMenu *menuProfile = item->GetSubMenu();
-
-  wxASSERT(place && menuTune && menuProfile);
 
   int itemid;
 
   switch(event.GetId()) {
-  case ID_ITT_TYPE:
-    itemid = menuProfile->FindItem(event.GetString());
-    wxASSERT(itemid != wxNOT_FOUND);
-    menuProfile->Check(itemid,true);
-    break;
   case ID_PALETTE_TYPE:
     if( menuPalette ) {
       itemid = menuPalette->FindItem(event.GetString());
@@ -1270,13 +976,23 @@ void MuniView::OnTuneFine(MuniTuneEvent& event)
       menuPalette->Check(itemid,true);
     }
     break;
-  case ID_PALETTE_NEGATIVE:
-    if( menuPalette )
-      menuTune->Check(ID_MENU_NEGATIVE,event.GetInt());
+  case ID_PALETTE_INVERSE:
+    if( menuTune ) {
+      palinv = event.GetInt();
+      menuTune->Check(ID_MENU_INVERSE,event.GetInt());
+    }
     break;
-  }
-
-  onreset = event.GetId() == ID_RESET;
+  case ID_ZOOM:
+    if( zoomctrl ) {
+     wxString a;
+     if( event.shrink )
+       a.Printf("1:%d",event.n);
+     else
+       a.Printf("%d:1",event.n);
+     zoomctrl->SetValue(a);
+    }
+    break;
+  } // case
 
   wxQueueEvent(place,event.Clone());
 }
@@ -1406,7 +1122,8 @@ void MuniView::OnMenuStructure(wxCommandEvent& event)
     if( viewid[i] == event.GetId() )
       hdu = i;
 
-  if( hdu == hdusel || hdu < -1 || hdu >= static_cast<int>(meta.HduCount()) ) return;
+  if( hdu == hdusel || hdu < -1 || hdu >= static_cast<int>(meta.HduCount()) )
+    return;
 
   hdusel = hdu;
 
@@ -1515,7 +1232,6 @@ void MuniView::HduSelect(int h)
     case HDU_IMAGE:  CreateDisplay(); break;
     case HDU_TABLE:  CreateGrid();    break;
     case HDU_HEAD:   CreateHead();    break;
-    default:         CreateResume();  break;
     }
 
     if( hdusel == 0 ) {
@@ -1542,8 +1258,6 @@ void MuniView::HduSelect(int h)
   wxASSERT(idm != wxNOT_FOUND);
   wxMenu *menuWin = GetMenuBar()->GetMenu(idm);
   wxASSERT(menuWin);
-  tbar->EnableTool(wxID_BACKWARD,backward);
-  tbar->EnableTool(wxID_FORWARD,forward);
   menuWin->Enable(wxID_BACKWARD,backward);
   menuWin->Enable(wxID_FORWARD,forward);
 
@@ -1581,7 +1295,6 @@ void MuniView::CreateHead()
 {
   ReplacePlace(new MuniHead(this,config));
   static_cast<MuniHead *>(place)->SetHdu(fits.Hdu(hdusel));
-  GetToolBar()->EnableTool(wxID_HOME,true);
   GetToolBar()->EnableTool(ID_INFO,true);
 
   wxMenu *menuHead = new wxMenu;
@@ -1593,7 +1306,6 @@ void MuniView::CreateHead()
 
 void MuniView::DeleteHead()
 {
-  GetToolBar()->EnableTool(wxID_HOME,false);
   GetToolBar()->EnableTool(ID_INFO,false);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnHeader,this,ID_INFO);
   wxMenuBar *menuBar = GetMenuBar();
@@ -1608,7 +1320,6 @@ void MuniView::CreateGrid()
 {
   ReplacePlace(new MuniGrid(this,config));
   static_cast<MuniGrid *>(place)->SetHdu(fits.Hdu(hdusel));
-  GetToolBar()->EnableTool(wxID_HOME,true);
   GetToolBar()->EnableTool(ID_INFO,true);
 
   wxMenu *menuGrid = new wxMenu;
@@ -1620,7 +1331,6 @@ void MuniView::CreateGrid()
 
 void MuniView::DeleteGrid()
 {
-  GetToolBar()->EnableTool(wxID_HOME,false);
   GetToolBar()->EnableTool(ID_INFO,false);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnHeader,this,ID_INFO);
 
@@ -1636,7 +1346,8 @@ void MuniView::CreateDisplay()
 {
   ReplacePlace(new MuniDisplay(this,config));
   static_cast<MuniDisplay *>(place)->SetHdu(fits.Hdu(hdusel),
-					    meta.Hdu(hdusel).GetIcon());
+					    meta.Hdu(hdusel).GetIcon(),
+					    palauto);
 
   // menus
   menuView->AppendCheckItem(ID_DETAIL,"Detail Panel");
@@ -1645,24 +1356,9 @@ void MuniView::CreateDisplay()
   menuView->AppendCheckItem(ID_GRID,"Coordinate Grid");
   menuView->AppendCheckItem(ID_SOURCES,"Detected Sources\tCtrl+G");
 
-
-  wxMenu *menuProfile = new wxMenu;
-  wxArrayString profiles = FitsItt::Type_str();
-  for(size_t i = 0; i < profiles.GetCount(); i++) {
-    menuProfile->AppendRadioItem(wxID_ANY,profiles.Item(i));
-    long id = menuProfile->FindItem(profiles.Item(i));
-    Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuItt,this,id);
-  }
-
-  wxString label;
-  label = FitsItt::Type_str(config->display_itt);
-  int id = menuProfile->FindItem(label);
-  wxASSERT(id != wxNOT_FOUND);
-  menuProfile->Check(id,true);
-
   wxMenu *menuImage = new wxMenu;
   menuImage->Append(ID_INFO,"Header");
-  //  menuImage->Append(ID_PHOT,"Photometry");
+  //  menuImage->Append(ID_PHOTOMETRY,"Photometry");
   //  menuImage->Append(ID_ASTROMETRY,"Astrometry");
   //  menuImage->Append(ID_CALIBR,"Calibrate");
   //  menuImage->AppendSeparator();
@@ -1674,12 +1370,9 @@ void MuniView::CreateDisplay()
   menuTune->Append(wxID_ZOOM_IN,"Zoom\tCtrl++");
   menuTune->Append(wxID_ZOOM_OUT,"Shrink\tCtrl+-");
   menuTune->AppendSeparator();
-  menuTune->Append(ID_CYCLE_ITT,"&Cycle Tones\tCtrl+T",
-  		    "Cycle throughout Tone curves");
-  menuTune->AppendSubMenu(menuProfile,"&" TONE_CURVES);
-  menuTune->AppendSeparator();
 
-  if( !fits.Hdu(hdusel).IsColor() ) {
+  FitsArray hdu(fits.Hdu(hdusel));
+  if( ! hdu.IsColor() ) {
 
     wxMenu *menuPalette = new wxMenu;
     wxArrayString pals = FitsPalette::Type_str();
@@ -1689,16 +1382,19 @@ void MuniView::CreateDisplay()
       Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuPal,this,id);
     }
 
-    label = FitsPalette::Type_str(config->display_pal);
-    id = menuPalette->FindItem(label);
+    wxString label(FitsPalette::Type_str(config->display_pal));
+    int id = menuPalette->FindItem(label);
     wxASSERT(id != wxNOT_FOUND);
     menuPalette->Check(id,true);
 
-    menuTune->AppendCheckItem(ID_MENU_NEGATIVE,"Negative");
     menuTune->Append(ID_CYCLE_PAL,"&Cycle Palette\tCtrl+P",
-		     "Cycle throughout Color Palettes");
+		     "Cycle throughout Colour Palettes");
     menuTune->AppendSubMenu(menuPalette,"&" COLOR_PALETTES);
+    menuTune->AppendCheckItem(ID_MENU_INVERSE,"Inverse");
+    menuTune->AppendCheckItem(ID_MENU_PALAUTO,"Auto Select");
     menuTune->AppendSeparator();
+    menuTune->Check(ID_MENU_INVERSE,palinv);
+    menuTune->Check(ID_MENU_PALAUTO,palauto);
   }
 
   menuTune->Append(ID_TUNE,"&Tune...");
@@ -1721,16 +1417,30 @@ void MuniView::CreateDisplay()
   wxToolBar *tbar = GetToolBar();
   wxASSERT(tbar);
 
-  int n = tbar->GetToolPos(ID_INFO) + 3;
+  int n = tbar->GetToolPos(ID_INFO) + 2;
 
-  tbar->InsertTool(n++,wxID_ZOOM_100,"Original",ico.Icon("zoom-original"),
-		   wxNullBitmap,wxITEM_NORMAL,"Original Size");
-  tbar->InsertTool(n++,wxID_ZOOM_FIT,"Fit",ico.Icon("zoom-fit-best"),
-		   wxNullBitmap,wxITEM_NORMAL,"Fit to Size");
   tbar->InsertTool(n,ID_TUNE,"Tune",ico.Icon("preferences-desktop"),
 		   wxNullBitmap,wxITEM_NORMAL,"Fine tunning controls");
 
-  tbar->EnableTool(wxID_HOME, true);
+  wxArrayString zoomlist;
+  zoomlist.Add("1:8");
+  zoomlist.Add("1:6");
+  zoomlist.Add("1:4");
+  zoomlist.Add("1:3");
+  zoomlist.Add("1:2");
+  zoomlist.Add("1:1");
+  zoomlist.Add("2:1");
+  zoomlist.Add("4:1");
+
+  zoomctrl = new wxComboBox(tbar,ID_TOOLZOOM,"",wxDefaultPosition,
+			    wxDefaultSize,zoomlist,wxCB_READONLY);
+  tbar->InsertControl(n+1,zoomctrl);
+
+  tbar->InsertTool(n+2,wxID_ZOOM_100,"Original",ico.Icon("zoom-original"),
+		   wxNullBitmap,wxITEM_NORMAL,"Original Size");
+  tbar->InsertTool(n+3,wxID_ZOOM_FIT,"Fit",ico.Icon("zoom-fit-best"),
+  		   wxNullBitmap,wxITEM_NORMAL,"Fit to Size");
+
   tbar->EnableTool(ID_INFO,true);
 
   tbar->Realize();
@@ -1751,23 +1461,25 @@ void MuniView::CreateDisplay()
     }
   }
 
+  Bind(EVT_TUNE,&MuniView::OnTuneFine,this);
+  Bind(wxEVT_COMBOBOX,&MuniView::OnToolZoom,this,ID_TOOLZOOM);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuZoom,this,wxID_ZOOM_100);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuZoom,this,wxID_ZOOM_FIT);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnCycleZoom,this,wxID_ZOOM_IN);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnCycleZoom,this,wxID_ZOOM_OUT);
-  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuNegative,this,ID_MENU_NEGATIVE);
-  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnCycleItt,this,ID_CYCLE_ITT);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnCyclePal,this,ID_CYCLE_PAL);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnTune,this,ID_TUNE);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnDetailPanel,this,ID_DETAIL);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnHeader,this,ID_INFO);
-  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnPhotometry,this,ID_PHOTOMETRY);
-  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnAstrometry,this,ID_ASTROMETRY);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnCalibrate,this,ID_CALIBR);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnShowGrid,this,ID_GRID);
   Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnShowSources,this,ID_SOURCES);
-  Bind(EVT_TUNE,&MuniView::OnTuneFine,this);
-  //  Bind(EVT_DIALOG,&MuniView::OnPhotometry,this,ID_PHOT);
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnPhotometry,this,ID_PHOTOMETRY);
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnAstrometry,this,ID_ASTROMETRY);
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuInverse,this,
+       ID_MENU_INVERSE);
+  Bind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuPalauto,this,
+       ID_MENU_PALAUTO);
 
   double xpix, ypix, alpha, delta, scale, angle, reflex;
   if( fits.Hdu(hdusel).GetWCS(xpix, ypix, alpha, delta, scale, angle, reflex) ){
@@ -1781,38 +1493,47 @@ void MuniView::CreateDisplay()
     wxASSERT(dynamic_cast<MuniDisplay *>(place));
     dynamic_cast<MuniDisplay *>(place)->ShowSources(config->display_sources,fits);
   }
+
+  MuniTuneEvent ev(EVT_TUNE,ID_PALETTE_INVERSE);
+  ev.SetInt(palinv);
+  wxQueueEvent(place,ev.Clone());
 }
 
 void MuniView::DeleteDisplay()
 {
   wxToolBar *tbar = GetToolBar();
   wxASSERT(tbar);
+
+  tbar->DeleteTool(ID_TOOLZOOM);
   tbar->DeleteTool(ID_TUNE);
   tbar->DeleteTool(wxID_ZOOM_FIT);
   tbar->DeleteTool(wxID_ZOOM_100);
 
   tbar->EnableTool(ID_INFO,false);
-  tbar->EnableTool(wxID_HOME,false);
 
   tbar->Realize();
 
+  Unbind(EVT_TUNE,&MuniView::OnTuneFine,this);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuZoom,this,wxID_ZOOM_100);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuZoom,this,wxID_ZOOM_FIT);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnCycleZoom,this,wxID_ZOOM_IN);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnCycleZoom,this,wxID_ZOOM_OUT);
-  Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuNegative,this,ID_MENU_NEGATIVE);
-  Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnCycleItt,this,ID_CYCLE_ITT);
+  Unbind(wxEVT_COMBOBOX,&MuniView::OnToolZoom,this,ID_TOOLZOOM);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnCyclePal,this,ID_CYCLE_PAL);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnTune,this,ID_TUNE);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnDetailPanel,this,ID_DETAIL);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnHeader,this,ID_INFO);
-  Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnPhotometry,this,ID_PHOTOMETRY);
-  Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnAstrometry,this,ID_ASTROMETRY);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnCalibrate,this,ID_CALIBR);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnShowGrid,this,ID_GRID);
   Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnShowSources,this,ID_SOURCES);
-  Unbind(EVT_TUNE,&MuniView::OnTuneFine,this);
-  //  Unbind(EVT_DIALOG,&MuniView::OnPhotometry,this,ID_PHOT);
+  Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnPhotometry,this,
+	 ID_PHOTOMETRY);
+  Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnAstrometry,this,
+	 ID_ASTROMETRY);
+  Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuInverse,this,
+	 ID_MENU_INVERSE);
+  Unbind(wxEVT_COMMAND_MENU_SELECTED,&MuniView::OnMenuPalauto,this,
+	 ID_MENU_PALAUTO);
 
   wxMenuBar *menuBar = GetMenuBar();
   int id = menuBar->FindMenu(MENU_FRAME);
@@ -1859,11 +1580,6 @@ void MuniView::OnEnterWin(wxMouseEvent& event)
   place->SetFocus();
 }
 
-void MuniView::OnLeaveWin(wxMouseEvent& event)
-{
-  // ?????
-}
-
 void MuniView::OnSize(wxSizeEvent& event)
 {
   //  wxASSERT(place);
@@ -2002,39 +1718,3 @@ void MuniView::OnColoringFinish(wxCommandEvent& event)
 //   }
 
 // }
-
-void MuniView::OnConfigUpdated(wxCommandEvent& event)
-{
-  /*
-  MuniDisplay *d = GetDisplay();
-  if( d )
-    wxQueueEvent(d,event.Clone());
-  */
-}
-
-
-// --- FitsOpen -------------------
-
-FitsOpen::FitsOpen(wxEvtHandler *h, const wxString& fn):
-wxThread(wxTHREAD_DETACHED),filename(fn),handler(h)
-{
-  wxASSERT(handler);
-}
-
-FitsOpen::~FitsOpen()
-{
-  wxCriticalSectionLocker enter(static_cast<MuniView *>(handler)->loaderCS);
-  static_cast<MuniView *>(handler)->loader = 0;
-}
-
-wxThread::ExitCode FitsOpen::Entry()
-{
-  FitsFile fits(filename);
-
-  FitsOpenEvent ev(EVT_FITS_OPEN,ID_LOADER);
-  ev.filename = filename;
-  ev.fits = fits;
-  wxQueueEvent(handler,ev.Clone());
-
-  return (wxThread::ExitCode) 0;
-}
