@@ -3,7 +3,7 @@
   XMunipack
 
 
-  Copyright © 2009-2015, 2017 F.Hroch (hroch@physics.muni.cz)
+  Copyright © 2009-2015, 2017-8 F.Hroch (hroch@physics.muni.cz)
 
   This file is part of Munipack.
 
@@ -102,17 +102,12 @@ private:
 
   MuniConfig *config;
   wxChoice *ocs;
-  wxSpinCtrl *temper;
-  wxTextCtrl *tu,*tv;
-  wxString uwhite,vwhite;
 
-  void Init();
   wxPanel *CreateGeneral(wxWindow *);
   wxPanel *CreateKeywords(wxWindow *);
   wxPanel *CreateColors(wxWindow *);
 
   void OnClose(wxCloseEvent&);
-  void OnColorTemper(wxSpinEvent&);
   void OnColorReset(wxCommandEvent&);
   void OnIdle(wxIdleEvent&);
 
@@ -154,18 +149,38 @@ private:
 class FitsOpen: public wxThread
 {
 public:
-  FitsOpen(wxEvtHandler *,const wxString&);
+  FitsOpen(wxEvtHandler *,const wxString&, int);
   virtual ~FitsOpen();
 
 private:
 
-  wxString filename;
   wxEvtHandler *handler;
+  wxString filename;
+  int icon_size;
 
   ExitCode Entry();
 
 };
 
+class FitsExport: public wxThread
+{
+public:
+  FitsExport(wxEvtHandler *, const FitsArray&, const wxString&, const FitsTone&,
+	     const FitsItt&, const FitsPalette&, const FitsColor&);
+  virtual ~FitsExport();
+
+private:
+
+  wxEvtHandler *handler;
+  FitsArray array;
+  wxString savename;
+  FitsTone tone;
+  FitsItt itt;
+  FitsPalette pal;
+  FitsColor colour;
+
+  ExitCode Entry();
+};
 
 
 class MuniFileProperties: public wxDialog
@@ -356,22 +371,6 @@ private:
 // };
 
 
-class MuniLUTus: public wxPanel
-{
-public:
-  MuniLUTus(wxWindow *);
-  wxSize DoGetBestSize() const;
-
-  void SetPalette(const FitsPalette&);
-
-private:
-
-  FitsPalette pal;
-
-  void OnPaint(wxPaintEvent&);
-  void OnSize(wxSizeEvent&);
-  void Create();
-};
 
 
 class MuniThumbCanvas: public wxWindow
@@ -419,15 +418,17 @@ protected:
 class MuniPhotometryLayer
 {
 public:
-  MuniPhotometryLayer(): gold(wxColour(255,215,0,240)),DarkOrange2(wxColour(238,118,0)) {}
+ MuniPhotometryLayer(): gold(wxColour(255,215,0,240)),DarkOrange2(wxColour(238,118,0)),hwhm(1) {}
   MuniLayer GetLayer() const;
   bool IsEmpty() const { return objects.empty(); }
   void DrawObjects(const std::vector<double>&,const std::vector<double>&,const std::vector<double>&);
+  void SetHWHM(double x) { hwhm = x; }
 
 private:
 
   const wxColour gold,DarkOrange2;
   std::vector<wxObject *> objects;
+  double hwhm;
 
 };
 
@@ -797,18 +798,17 @@ class MuniDisplay: public wxWindow
 public:
   MuniDisplay(wxWindow *w, MuniConfig *);
 
-  bool SetHdu(const FitsHdu&, const wxImage&);
+  bool SetHdu(const FitsHdu&, const wxImage&, bool);
   //  virtual void SetOverlay(wxInputStream&);
   virtual void SetStars(FitsTable&);
-  bool IsRendering() const;
   void StopRendering();
-  double GetZoom() const;
-  double GetFitZoom() const;
+  int GetFitShrink() const;
+  FitsZoom GetZoom() const;
+  FitsTone GetTone() const;
   FitsItt GetItt() const;
   FitsPalette GetPalette() const;
   FitsColor GetColor() const;
   wxImage GetImage();
-  bool GetFit() const;
   void ShowPanel(bool);
   void ShowTune(bool =true);
   void ShowGrid(bool =true);
@@ -893,36 +893,52 @@ private:
   void EditPaste(wxCommandEvent&);
 };
 
+class MuniSplashing: public wxWindow
+{
+private:
+
+  wxBitmap logo;
+  wxAnimationCtrl *anim;
+
+public:
+  MuniSplashing(wxWindow *, const MuniConfig *);
+  void Play();
+  void Stop();
+
+};
+
+
+
 class MuniView: public wxFrame
 {
 public:
   MuniView(wxWindow *,MuniConfig *);
-  virtual ~MuniView();
   void LoadFile(const wxString&);
   void SaveFile(const wxString&);
   void LoadMeta(const FitsMeta&);
   void LoadFileBackup(const wxString&, const wxString&);
   bool IsModified() const;
 
-
-  // private ?
-  FitsOpen *loader;
-  wxCriticalSection loaderCS;
-
 private:
 
+  friend class FitsOpen;
+  friend class FitsExport;
+
   MuniConfig *config;
   FitsFile fits;
   FitsMeta meta;
   int hdusel;
+  bool palinv, palauto;
   std::vector<long> viewid;
-  wxString result;
   wxArrayString backup;
 
+  wxThread *loader;
+  wxCriticalSection loaderCS;
+
   wxMenu *menuFile,*menuView,*menuWin;
   wxComboCtrl* structtree;
+  wxComboBox *zoomctrl;
   wxWindow *place;
-  bool onreset;
   MuniColoring *coloring;
   wxLogWindow *console;
 
@@ -971,18 +987,17 @@ private:
   void FileProperties(wxCommandEvent& WXUNUSED(event));
   void OnPreferences(wxCommandEvent& WXUNUSED(event));
   void ShowToolbar(wxCommandEvent&);
-  void OnMenuNegative(wxCommandEvent&);
-  void OnMenuItt(wxCommandEvent&);
+  void OnMenuPalauto(wxCommandEvent&);
+  void OnMenuInverse(wxCommandEvent&);
   void OnMenuPal(wxCommandEvent&);
   void OnMenuStructure(wxCommandEvent&);
   void OnSelectFrame(wxCommandEvent&);
   void OnMenuZoom(wxCommandEvent&);
+  void OnToolZoom(wxCommandEvent&);
   void OnCycleZoom(wxCommandEvent&);
-  void OnCycleItt(wxCommandEvent&);
   void OnCyclePal(wxCommandEvent&);
   void OnDetailPanel(wxCommandEvent&);
   void OnEnterWin(wxMouseEvent&);
-  void OnLeaveWin(wxMouseEvent&);
   //  void SaveAsText(wxString);
   void FileSave(wxCommandEvent& WXUNUSED(event));
   void FileExport(wxCommandEvent& WXUNUSED(event));
@@ -1000,12 +1015,9 @@ private:
   */
 
   void LoadStart(const wxString&);
-  //  void OnFileLoad(wxCommandEvent&);
-  void OnUpdateFit(wxUpdateUIEvent&);
-  void OnUpdateNFit(wxUpdateUIEvent&);
+  void ExportStart(const wxString&);
   void OnUpdateInfos(wxUpdateUIEvent&);
 
-  void OnConfigUpdated(wxCommandEvent&);
   void OnResumeHdu(MuniHduEvent&);
   int Unsaved(const wxString&);
   void RemoveBackup();
@@ -1050,11 +1062,11 @@ public:
 
   void SetEventHandler(wxEvtHandler *);
 
-  wxThread *loader;
-  wxCriticalSection loaderCS;
 
 private:
 
+  friend class xArchiveThread;
+
   wxString GetFullPath(const wxString& =wxEmptyString) const;
   wxString DissolveRoot(const wxString&) const;
   long NextAvailableIndex() const;
@@ -1069,6 +1081,8 @@ private:
   const wxString sep;
 
   wxEvtHandler *handler;
+  wxThread *loader;
+  wxCriticalSection loaderCS;
 };
 
 
@@ -1155,15 +1169,17 @@ public:
 
   void AddItem(const FitsMeta&);
 
-  MetaRender *metarender;
-  wxCriticalSection metarenderCS;
-
 protected:
 
+  friend class MetaRender;
+
   MuniConfig *config;
   wxImageList *thumbs;
   std::vector<FitsMeta> flist,addlist,dellist;
   std::queue<FitsMeta> metas;
+  wxThread *metarender;
+  wxCriticalSection metarenderCS;
+
 
   void OnRightClick(wxListEvent&);
   void OnView(wxCommandEvent& WXUNUSED(event));
